{
  Drawing.pas - freeware unit for quick and best drawing graphics effects

  Author : Rustam T. Kafarov
  E-mail : rustamkafaroc@hotmail.com
  ICQ #  : 67855579
  Copyright  2001

}



unit Drawing;

interface

Uses Windows, Graphics, Classes, Math, SysUtils;

Type
  TAngle = 0..180;
  TGradType = ( gtNone, gtLinear, gtLinearDouble, gtRadial, gtRadialDouble, gtSquare, gtSquareDouble, gtConic );
  T3DStyle = ( stShadow, stExtrude, stGradExtrude );
  TRectPoints = Array[0..4] Of TPoint;

Type
  TGradientOrientation = (goHorizontal, goVertical, goHorzCenter, goVertCenter, goRadial );

  Function IsEmptyRect( ARect : TRect ) : Boolean;

  Procedure CorrectRect( var ARect : TRect );

  Function RectIntersection( Rect1, Rect2 : TRect ) : TRect;

  Function RotateRectToRGN( ARect : TRect; Alpha : TAngle; Var RectPoints : TRectPoints ) : HRGN;

  Procedure DrawRotatedRectangle( ACanvas : TCanvas; ARect : TRect; Angle : TAngle  );

  Procedure GradientRect( ACanvas: TCanvas; ARect, ClipRect : TRect;  Color1, Color2 : TColor;
                          GradType : TGradType; Params : Byte );


implementation

Const
  MaxColors = 128;

{====== Special THANKS to my friend Kambiz R. Khojasteh for this function! =============}
{                                   E-mail: khojasteh@mail.com                          }
{                                   Web: http://www.crosswinds.net/~khojasteh/          }

function IntersectLineEllipse(Angle: Extended; R: TRect; Rev: Boolean): TPoint;
var
  A2, B2, M, T: Extended;
  Xc, Yc, X, Y: Extended;
begin
  Xc := (R.Left + R.Right) / 2;
  Yc := (R.Top + R.Bottom) / 2;
  if (Abs(Angle) <> Pi / 2) and (Angle <> 0) and (Abs(Angle) <> Pi) then
  begin
    M := Tan(Angle);
    A2 := Sqr((R.Right - R.Left) / 2);
    B2 := Sqr((R.Bottom - R.Top) / 2);
    T := B2 + A2 * Sqr(M);
    if ((Abs(Angle) > Pi / 2) and not Rev) or
       ((Abs(Angle) < Pi / 2) and Rev) then
      X := Sqrt(T * (A2 * B2)) / T
    else
      X := -Sqrt(T * (A2 * B2)) / T;
    Y := M * X;
    Result := Point(Round(X+Xc), Round(Y+Yc));
  end
  else if Abs(Angle) = Pi / 2 then
  begin
    if ((Angle < 0) and not Rev) or ((Angle > 0) and Rev) then
      Result := Point(Round(Xc), R.Bottom)
    else
      Result := Point(Round(Xc), R.Top);
  end
  else
  begin
    if ((Angle = 0) and not Rev) or ((Abs(Angle) = Pi) and Rev) then
      Result := Point(R.Left, Round(Yc))
    else
      Result := Point(R.Right, Round(Yc));
  end;
end;

{================================}


Function IsEmptyRect( ARect : TRect ) : Boolean;
Begin
  Result := (ARect.Left = 0) And (ARect.Top = 0) And (ARect.Right = 0) And (ARect.Bottom = 0);
End;

Procedure CorrectRect( var ARect : TRect );
Var
  Temp : Integer;
Begin
  If ARect.Left > ARect.Right
    Then Begin
           Temp := ARect.Right;
           ARect.Left := ARect.Right;
           ARect.Right := Temp;
         End;
  If ARect.Top > ARect.Bottom
    Then Begin
           Temp := ARect.Top;
           ARect.Top := ARect.Bottom;
           ARect.Bottom := Temp;
         End;
End;

Function RectIntersection( Rect1, Rect2 : TRect ) : TRect;
Var
  X1_1, Y1_1, X2_1, Y2_1 : Integer;
  X1_2, Y1_2, X2_2, Y2_2 : Integer;
  X1_3, Y1_3, X2_3, Y2_3 : Integer;
Begin
  If Rect1.Left < Rect1.Right
    Then Begin
           X1_1 := Rect1.Left;
           X2_1 := Rect1.Right;
         End
    Else Begin
           X1_1 := Rect1.Right;
           X2_1 := Rect1.Left;
         End;
  If Rect1.Top < Rect1.Bottom
    Then Begin
           Y1_1 := Rect1.Top;
           Y2_1 := Rect1.Bottom;
         End
    Else Begin
           Y1_1 := Rect1.Bottom;
           Y2_1 := Rect1.Top;
         End;

  If Rect2.Left < Rect2.Right
    Then Begin
           X1_2 := Rect2.Left;
           X2_2 := Rect2.Right;
         End
    Else Begin
           X1_2 := Rect2.Right;
           X2_2 := Rect2.Left;
         End;
  If Rect2.Top < Rect2.Bottom
    Then Begin
           Y1_2 := Rect2.Top;
           Y2_2 := Rect2.Bottom;
         End
    Else Begin
           Y1_2 := Rect2.Bottom;
           Y2_2 := Rect2.Top;
         End;

  If X1_1 > X1_2
    Then If X1_1 < X2_2
           THen X1_3 := X1_1
           Else Begin
                  Result := Rect( 0, 0, 0, 0 );
                  Exit;
                End
    Else If X1_2 < X2_1
           Then X1_3 := X1_2
           Else Begin
                  Result := Rect( 0, 0, 0, 0 );
                  Exit;
                End;
  If Y1_1 > Y1_2
    Then If Y1_1 < Y2_2
           THen Y1_3 := Y1_1
           Else Begin
                  Result := Rect( 0, 0, 0, 0 );
                  Exit;
                End
    Else If Y1_2 < Y2_1
           Then Y1_3 := Y1_2
           Else Begin
                  Result := Rect( 0, 0, 0, 0 );
                  Exit;
                End;

  If X2_1 < X2_2
    Then If X2_1 > X1_2
           THen X2_3 := X2_1
           Else Begin
                  Result := Rect( 0, 0, 0, 0 );
                  Exit;
                End
    Else If X2_2 > X1_1
           Then X2_3 := X2_2
           Else Begin
                  Result := Rect( 0, 0, 0, 0 );
                  Exit;
                End;
  If Y2_1 < Y2_2
    Then If Y2_1 > Y1_2
           THen Y2_3 := Y2_1
           Else Begin
                  Result := Rect( 0, 0, 0, 0 );
                  Exit;
                End
    Else If Y2_2 > Y1_1
           Then Y2_3 := Y2_2
           Else Begin
                  Result := Rect( 0, 0, 0, 0 );
                  Exit;
                End;

  Result := Rect( X1_3, Y1_3, X2_3, Y2_3 );
End;

Function RotatePoint( Point : TPoint; Angle : SmallInt ) : TPoint;
Var
  NewPoint : TPoint;
  Alpha    : Extended;
Begin
  Alpha := (Angle * Pi) / 180;
  NewPoint.X := Round( Point.Y*Sin( Alpha ) + Point.X*Cos( Alpha ) );
  NewPoint.Y := Round( Point.Y*Cos( Alpha ) - Point.X*Sin( Alpha ) );
  Result := NewPoint;
End;

Function RotateRectToRGN( ARect : TRect; Alpha : TAngle; Var RectPoints : TRectPoints ) : HRGN;
Var
  I : Byte;
Begin
  RectPoints[0].X := - (Abs(ARect.Right - ARect.Left) Div 2 );
  RectPoints[0].Y := - (Abs(ARect.Bottom - ARect.Top) Div 2 );
  RectPoints[1].X := (Abs(ARect.Right - ARect.Left) Div 2 );
  RectPoints[1].Y := - (Abs(ARect.Bottom - ARect.Top) Div 2 );
  RectPoints[2].X := (Abs(ARect.Right - ARect.Left) Div 2 );
  RectPoints[2].Y := (Abs(ARect.Bottom - ARect.Top) Div 2 );
  RectPoints[3].X := - (Abs(ARect.Right - ARect.Left) Div 2 );
  RectPoints[3].Y := (Abs(ARect.Bottom - ARect.Top) Div 2 );
  RectPoints[4].X := RectPoints[0].X;
  RectPoints[4].Y := RectPoints[0].Y;

  For I := 0 To 4 Do
    Begin
      RectPoints[I] := RotatePoint( RectPoints[I], -Alpha );
    End;

  RectPoints[0].X := ARect.Left + (Abs(ARect.Right - ARect.Left) Div 2 ) + RectPoints[0].X;
  RectPoints[0].Y := ARect.Top + (Abs(ARect.Bottom - ARect.Top) Div 2 ) + RectPoints[0].Y;
  RectPoints[1].X := ARect.Left + (Abs(ARect.Right - ARect.Left) Div 2 ) + RectPoints[1].X;
  RectPoints[1].Y := ARect.Top + (Abs(ARect.Bottom - ARect.Top) Div 2 ) + RectPoints[1].Y;
  RectPoints[2].X := ARect.Left + (Abs(ARect.Right - ARect.Left) Div 2 ) + RectPoints[2].X;
  RectPoints[2].Y := ARect.Top + (Abs(ARect.Bottom - ARect.Top) Div 2 ) + RectPoints[2].Y;
  RectPoints[3].X := ARect.Left + (Abs(ARect.Right - ARect.Left) Div 2 ) + RectPoints[3].X;
  RectPoints[3].Y := ARect.Top + (Abs(ARect.Bottom - ARect.Top) Div 2 ) + RectPoints[3].Y;
  RectPoints[4].X := RectPoints[0].X;
  RectPoints[4].Y := RectPoints[0].Y;

//  Result := CreatePolygonRGN( RectPoints, 5, WINDING );
End;

Procedure DrawRotatedRectangle( ACanvas : TCanvas; ARect : TRect; Angle : TAngle  );
Var
  P : TRectPoints;
//  R : HRGN;
Begin
  RotateRectToRGN( ARect, Angle, P );
//  DeleteObject( R );
  ACanvas.Polygon( P );
End;

Procedure LinearGradient( ACanvas : TCanvas; ARect: TRect; ClipRect : TRect;
                          Color1, Color2 : TColor; Angle : TAngle );
Var
  RealRect   : TRect; // 
  Width      : Integer; //  
  Height     : Integer;
  Shift      : Integer;
  Times      : Integer;
  RealWidth  : Integer; //  
  RealHeight : Integer;
  RealShift  : Integer;
  RealTimes  : Integer;
  FirstColor : Integer;
  LastColor  : Integer;
  Red_1      : Byte;
  Green_1    : Byte;
  Blue_1     : Byte;
  Red_2      : Byte;
  Green_2    : Byte;
  Blue_2     : Byte;
  Red_Mix    : Byte;
  Green_Mix  : Byte;
  Blue_Mix   : Byte;
  I          : Integer;
  MixColor   : TColor;
  Weight1    : Integer;
  Weight2    : Integer;
  Alpha      : Extended;
Begin
  FirstColor := ColorToRGB(Color1);
  Red_1 := GetRValue(FirstColor);
  Green_1 := GetGValue(FirstColor);
  Blue_1 := GetBValue(FirstColor);
  LastColor := ColorToRGB(Color2);
  Red_2 := GetRValue(LastColor);
  Green_2 := GetGValue(LastColor);
  Blue_2 := GetBValue(LastColor);

  Width := Abs( ARect.Right - ARect.Left );
  Height := Abs( ARect.Bottom - ARect.Top );

  RealRect := RectIntersection( ARect, ClipRect );
  If IsEmptyRect( RealRect )
    Then Exit;

  RealWidth := Abs( RealRect.Right - RealRect.Left );
  RealHeight := Abs( RealRect.Bottom - RealRect.Top );
  Case Angle Of
  0..44    : Begin
               Alpha := Angle*Pi/180;
               Shift := Round(Width*TAN(Alpha));
               Times := Height + Shift;
               RealShift := Round(RealWidth*TAN(Alpha));
               RealTimes := RealHeight + RealShift;
             End;
  45..90   : Begin
               Alpha := (90-Angle)*Pi/180;
               Shift := Round(Height*TAN(Alpha));
               Times := Width + Shift;
               RealShift := Round(RealHeight*TAN(Alpha));
               RealTimes := RealWidth + RealShift;
             End;
  91..134  : Begin
               Alpha := (Angle-90)*Pi/180;
               Shift := Round(Height*TAN(Alpha));
               Times := Width + Shift;
               RealShift := Round(RealHeight*TAN(Alpha));
               RealTimes := RealWidth + RealShift;
             End;
  135..180 : Begin
               Alpha := (180-Angle)*Pi/180;
               Shift := Round(Width*TAN(Alpha));
               Times := Height + Shift;
               RealShift := Round(RealWidth*TAN(Alpha));
               RealTimes := RealHeight + RealShift;
             End;
  Else Exit;
  End;{ case }

  If RealTimes = 0
    Then Exit;

  ACanvas.Pen.Mode := pmCopy;
  ACanvas.Pen.Style := psSolid;
  ACanvas.Pen.Width := 2;
  For I := 0 To RealTimes Do
    Begin
      Weight1 := Times-I;
      Weight2 := I;
      Red_Mix := ( Red_1*Weight1 + Red_2*Weight2 ) Div Times;
      Green_Mix := ( Green_1*Weight1 + Green_2*Weight2 ) Div Times;
      Blue_Mix := ( blue_1*Weight1 + Blue_2*Weight2 ) Div Times;
      MixColor := RGB( Red_Mix, Green_Mix, Blue_Mix );
      With ACanvas Do
        Begin
          Pen.Color := MixColor;
          Case Angle Of
          0..44    : Begin
                       MoveTo( RealRect.Left, RealRect.Top - RealShift + I );
                       LineTo( RealRect.Left + RealWidth, RealRect.Top + I );
                     End;
          45..89   : Begin
                       MoveTo( RealRect.Left + RealWidth - I, RealRect.Top );
                       LineTo( RealRect.Left + RealWidth + RealShift - I - 1, RealRect.Top + RealHeight );
                      End;
          90..134  : Begin
                       MoveTo( RealRect.Left + RealWidth + RealShift - I - 1, RealRect.Top );
                       LineTo( RealRect.Left + RealWidth - I, RealRect.Top + RealHeight );
                     End;
          135..180 : Begin
                       MoveTo( RealRect.Left, RealRect.Top + RealHeight - I + RealShift );
                       LineTo( RealRect.Left + RealWidth, RealRect.Top + RealHeight - I );
                     End;
          End;{ case }
        End;
    End;
End;

Procedure LinearDoubleGradient( ACanvas : TCanvas; ARect: TRect; ClipRect : TRect;
                                Color1, Color2 : TColor; Angle : TAngle );
Var
  DrawRect   : TRect; // 
  RealRect   : TRect; // 
  Width      : Integer; //  
  Height     : Integer;
  Shift      : Integer;
  Times      : Integer;
  RealWidth  : Integer; //  
  RealHeight : Integer;
  RealShift  : Integer;
  RealTimes  : Integer;
  FirstColor : Integer;
  LastColor  : Integer;
  Red_1      : Byte;
  Green_1    : Byte;
  Blue_1     : Byte;
  Red_2      : Byte;
  Green_2    : Byte;
  Blue_2     : Byte;
  Red_Mix    : Byte;
  Green_Mix  : Byte;
  Blue_Mix   : Byte;
  I, Half    : Integer;
  MixColor   : TColor;
  Weight1    : Integer;
  Weight2    : Integer;
  Alpha      : Real;
Begin
  FirstColor := ColorToRGB(Color1);
  Red_1 := GetRValue(FirstColor);
  Green_1 := GetGValue(FirstColor);
  Blue_1 := GetBValue(FirstColor);
  LastColor := ColorToRGB(Color2);
  Red_2 := GetRValue(LastColor);
  Green_2 := GetGValue(LastColor);
  Blue_2 := GetBValue(LastColor);

  Width := Abs( ARect.Right - ARect.Left );
  Height := Abs( ARect.Bottom - ARect.Top );

  RealRect := RectIntersection( ARect, ClipRect );
  If IsEmptyRect( RealRect )
    Then Exit;

  RealWidth := Abs( RealRect.Right - RealRect.Left );
  RealHeight := Abs( RealRect.Bottom - RealRect.Top );
  Case Angle Of
  0..44    : Begin
               Alpha := Angle*Pi/180;
               Shift := Round(Width*TAN(Alpha));
               Times := Height + Shift;
               RealShift := Round(RealWidth*TAN(Alpha));
               RealTimes := RealHeight + RealShift;
             End;
  45..90   : Begin
               Alpha := (90-Angle)*Pi/180;
               Shift := Round(Height*TAN(Alpha));
               Times := Width + Shift;
               RealShift := Round(RealHeight*TAN(Alpha));
               RealTimes := RealWidth + RealShift;
             End;
  91..134  : Begin
               Alpha := (Angle-90)*Pi/180;
               Shift := Round(Height*TAN(Alpha));
               Times := Width + Shift;
               RealShift := Round(RealHeight*TAN(Alpha));
               RealTimes := RealWidth + RealShift;
             End;
  135..180 : Begin
               Alpha := (180-Angle)*Pi/180;
               Shift := Round(Width*TAN(Alpha));
               Times := Height + Shift;
               RealShift := Round(RealWidth*TAN(Alpha));
               RealTimes := RealHeight + RealShift;
             End;
  Else Exit;
  End;{ case }
  If RealTimes = 0
    Then Exit;

  Half  := Times Div 2 + 1;
  ACanvas.Pen.Mode := pmCopy;
  ACanvas.Pen.Style := psSolid;
  ACanvas.Pen.Width := 2;
  For I := 0 To RealTimes Do
    Begin
      If I <= Half
        Then Begin
               Weight1 := Half - I;
               Weight2 := I;
             End
        Else Begin
               Weight1 := I - Half;
               Weight2 := Times - I;
             End;
      Red_Mix := ( Red_1*Weight1 + Red_2*Weight2 ) Div Half;
      Green_Mix := ( Green_1*Weight1 + Green_2*Weight2 ) Div Half;
      Blue_Mix := ( blue_1*Weight1 + Blue_2*Weight2 ) Div Half;
      MixColor := RGB( Red_Mix, Green_Mix, Blue_Mix );
      With ACanvas Do
        Begin
          Pen.Color := MixColor;
          Case Angle Of
          0..44    : Begin
                       MoveTo( RealRect.Left, RealRect.Top - RealShift + I );
                       LineTo( RealRect.Left + RealWidth, RealRect.Top + I );
                     End;
          45..89   : Begin
                       MoveTo( RealRect.Left + RealWidth - I, RealRect.Top );
                       LineTo( RealRect.Left + RealWidth + RealShift - I, RealRect.Top + RealHeight );
                      End;
          90..134  : Begin
                       MoveTo( RealRect.Left + RealWidth + RealShift - I, RealRect.Top );
                       LineTo( RealRect.Left + RealWidth - I, RealRect.Top + RealHeight );
                     End;
          135..180 : Begin
                       MoveTo( RealRect.Left, RealRect.Top + RealHeight - I + RealShift );
                       LineTo( RealRect.Left + RealWidth, RealRect.Top + RealHeight - I );
                     End;
          End;{ case }
        End;
    End;
End;

procedure RoundGradient(ACanvas: TCanvas; ARect, ClipRect : TRect; C1, C2: TColor; Params : Integer; Mode : Boolean );
Var
  Radius1, Radius2  : Integer;
  R1, R2, I, Colors : Integer;
  Center   : TPoint;
  StartRGB : Array [1..3] Of Byte;
  DeltaRGB : Array [1..3] Of Integer;
  Rot_Rect : TRect;
Begin
  C1 := ColorToRGB(C1);
  C2 := ColorToRGB(C2);
  StartRGB[1] := GetRValue(C1);
  StartRGB[2] := GetGValue(C1);
  StartRGB[3] := GetBValue(C1);
  DeltaRGB[1] := GetRValue(C2) - StartRGB[1];
  DeltaRGB[2] := GetGValue(C2) - StartRGB[2];
  DeltaRGB[3] := GetBValue(C2) - StartRGB[3];
  If (Params <> 0) And (Not Mode)
    Then Begin
           Radius1 := Round(Params*(ARect.Right - ARect.Left)/100);
           Radius2 := Round(Params*(ARect.Bottom - ARect.Top)/100) ;
         End
    Else Begin
           Radius1 := (ARect.Right - ARect.Left) Div 2;
           Radius2 := (ARect.Bottom - ARect.Top) Div 2;
         End;
  If Radius1 < Radius2
    Then Colors := Radius1
    Else Colors := Radius2;
  If Colors > MaxColors
    Then Colors := MaxColors
    Else If Colors < 2
          Then Colors := 2;
  Center := Point((ARect.Left + ARect.Right) Div 2, (ARect.Bottom + ARect.Top) Div 2);
  ACanvas.Pen.Style := psClear;
  ACanvas.Brush.Style := bsSolid;
  ACanvas.Brush.Color := C2;
  ACanvas.FillRect(ARect);
  For I := Colors-1 Downto 0 Do
    Begin
      ACanvas.Brush.Color := RGB( StartRGB[1] + MulDiv(I, DeltaRGB[1], Colors-1),
                                  StartRGB[2] + MulDiv(I, DeltaRGB[2], Colors-1),
                                  StartRGB[3] + MulDiv(I, DeltaRGB[3], Colors-1) );
      R1 := MulDiv(I+1, Radius1, Colors);
      R2 := MulDiv(I+1, Radius2, Colors);
      If Mode
        Then Begin
               R1 := R1 + Round(R1*Sin(Params * Pi / 180)/2);
               R2 := R2 + Round(R2*Sin(Params * Pi / 180)/2);
               Rot_Rect := Rect(Center.X - R1, Center.Y - R2, Center.X + R1, Center.Y + R2);
               DrawRotatedRectangle( ACanvas, Rot_Rect, Params );
             End
        Else Begin
               Rot_Rect := Rect(Center.X - R1, Center.Y - R2, Center.X + R1, Center.Y + R2);
               If ( R1-1 <= Sqrt(2)*Abs(ARect.Right - ARect.Left)/2 ) Or
                  ( R2-1 <= Sqrt(2)*Abs(ARect.Bottom - ARect.Top)/2 )
                 Then ACanvas.Ellipse( Rot_Rect )
             End;
    End;
End;

procedure RoundDoubleGradient(ACanvas: TCanvas; ARect, ClipRect: TRect; C1, C2: TColor; Params : Integer; Mode : Boolean );
Var
  LR, LG, LB, RR, RG, RB, MR, MG, MB : Integer; //Left, Right, and Mix RGBs
  LeftRGB, RightRGB                  : Integer;
  LeftWeight, RightWeight            : Integer;
  Radius1, Radius2, Times  : Integer;
  R1, R2, I, Colors : Integer;
  Center   : TPoint;
  Rot_Rect : TRect;
Begin
  LeftRGB := ColorToRGB(C1);
  LR := GetRValue(LeftRGB);
  LG := GetGValue(LeftRGB);
  LB := GetBValue(LeftRGB);
  RightRGB := ColorToRGB(C2);
  RR := GetRValue(RightRGB);
  RG := GetGValue(RightRGB);
  RB := GetBValue(RightRGB);
  If (Params <> 0) And (Not Mode)
    Then Begin
           Radius1 := Round(Params*(ARect.Right - ARect.Left)/100);
           Radius2 := Round(Params*(ARect.Bottom - ARect.Top)/100) ;
         End
    Else Begin
           Radius1 := (ARect.Right - ARect.Left);
           Radius2 := (ARect.Bottom - ARect.Top);
         End;
  If Radius1 < Radius2
    Then Colors := Radius1
    Else Colors := Radius2;
  If Colors > MaxColors
    Then Colors := MaxColors
    Else If Colors < 2
          Then Colors := 2;
  Center := Point((ARect.Left + ARect.Right) Div 2, (ARect.Bottom + ARect.Top) Div 2);
  ACanvas.Pen.Style := psClear;
  ACanvas.Pen.Color := C1;
  ACanvas.Brush.Style := bsSolid;
  ACanvas.Brush.Color := C1;
  ACanvas.FillRect(ARect);
  Times := Colors Div 2;
  For I := Colors DownTo Times Do
    Begin
      LeftWeight := I - Times;
      RightWeight := Colors - I;
      MR := (LR*LeftWeight+RR*RightWeight) Div Times;
      MG := (LG*LeftWeight+RG*RightWeight) Div Times;
      MB := (LB*LeftWeight+RB*RightWeight) Div Times;
      ACanvas.Brush.Color := RGB(MR, MG, MB);
      R1 := MulDiv( I+2, Radius1, 2*Colors );
      R2 := MulDiv( I+2, Radius2, 2*Colors );
      If Mode
        Then Begin
               R1 := R1 + Round(R1*Sin(Params * Pi / 180)/2);
               R2 := R2 + Round(R2*Sin(Params * Pi / 180)/2);
               Rot_Rect := Rect(Center.X - R1, Center.Y - R2, Center.X + R1, Center.Y + R2);
               DrawRotatedRectangle( ACanvas, Rot_Rect, Params );
             End
        Else Begin
               Rot_Rect := Rect(Center.X - R1, Center.Y - R2, Center.X + R1, Center.Y + R2);
               If ( R1-1 <= Sqrt(2)*Abs(ARect.Right - ARect.Left)/2 ) Or
                  ( R2-1 <= Sqrt(2)*Abs(ARect.Bottom - ARect.Top)/2 )
                 Then ACanvas.Ellipse( Rot_Rect );
             End;
    End;
  For I := Times DownTo 0 Do
    Begin
      LeftWeight := Times - I;
      RightWeight := I;
      MR := (LR*LeftWeight+RR*RightWeight) Div Times;
      MG := (LG*LeftWeight+RG*RightWeight) Div Times;
      MB := (LB*LeftWeight+RB*RightWeight) Div Times;
      ACanvas.Brush.Color := RGB(MR, MG, MB);
      R1 := MulDiv( I+2, Radius1, 2*Colors );
      R2 := MulDiv( I+2, Radius2, 2*Colors );
      If Mode
        Then Begin
               R1 := R1 + Round(R1*Sin(Params * Pi / 180)/2);
               R2 := R2 + Round(R2*Sin(Params * Pi / 180)/2);
               Rot_Rect := Rect(Center.X - R1, Center.Y - R2, Center.X + R1, Center.Y + R2);
               DrawRotatedRectangle( ACanvas, Rot_Rect, Params );
             End
        Else Begin
               Rot_Rect := Rect(Center.X - R1, Center.Y - R2, Center.X + R1, Center.Y + R2);
               If ( R1-1 <= Sqrt(2)*Abs(ARect.Right - ARect.Left)/2 ) Or
                  ( R2-1 <= Sqrt(2)*Abs(ARect.Bottom - ARect.Top)/2 )
                 Then ACanvas.Ellipse( Rot_Rect );
             End;
    End;
End;

Procedure ConicGradient( ACanvas : TCanvas; ARect, ClipRect : TRect; Color1, Color2 : TColor; Params : SmallInt );
Var
  Width      : Integer;
  Height     : Integer;
  Center     : TPoint;
  Times, I   : Integer;
  FirstColor : Integer;
  LastColor  : Integer;
  Red_1      : Byte;
  Green_1    : Byte;
  Blue_1     : Byte;
  Red_2      : Byte;
  Green_2    : Byte;
  Blue_2     : Byte;
  Red_Mix    : Byte;
  Green_Mix  : Byte;
  Blue_Mix   : Byte;
  MixColor   : TColor;
  Weight1    : Integer;
  Weight2    : Integer;
  Alpha      : Real;
Begin
  FirstColor := ColorToRGB(Color1);
  Red_1 := GetRValue(FirstColor);
  Green_1 := GetGValue(FirstColor);
  Blue_1 := GetBValue(FirstColor);
  LastColor := ColorToRGB(Color2);
  Red_2 := GetRValue(LastColor);
  Green_2 := GetGValue(LastColor);
  Blue_2 := GetBValue(LastColor);

  Width := Abs( ARect.Right - ARect.Left );
  Height := Abs( ARect.Bottom - ARect.Top );

  Center.X := ARect.Left + Width Div 2;
  Center.Y := ARect.Top + Height Div 2;

  With ACanvas Do
    Begin
      Pen.Width := 2;
      I := 0;
      While I <= Width Do
        Begin
          Weight1 := I;
          Weight2 := Width + Height - I;
          Red_Mix := ( Red_1*Weight1 + Red_2*Weight2 ) Div (Height+Width);
          Green_Mix := ( Green_1*Weight1 + Green_2*Weight2 ) Div (Height+Width);
          Blue_Mix := ( Blue_1*Weight1 + Blue_2*Weight2 ) Div (Height+Width);
          MixColor := RGB( Red_Mix, Green_Mix, Blue_Mix );
          Pen.Color := MixColor;
          MoveTo( Center.X, Center.Y );
          LineTo( ARect.Left + I, ARect.Top );

          Weight1 := Height + I;
          Weight2 := Width - I;
          Red_Mix := ( Red_1*Weight1 + Red_2*Weight2 ) Div (Height+Width);
          Green_Mix := ( Green_1*Weight1 + Green_2*Weight2 ) Div (Height+Width);
          Blue_Mix := ( Blue_1*Weight1 + Blue_2*Weight2 ) Div (Height+Width);
          MixColor := RGB( Red_Mix, Green_Mix, Blue_Mix );
          Pen.Color := MixColor;
          MoveTo( Center.X, Center.Y );
          LineTo( ARect.Left + I, ARect.Bottom );
          Inc( I );
        End;

      I := 0;
      While I <= Height Do
        Begin
          Weight1 := Width + I;
          Weight2 := Height - I;
          Red_Mix := ( Red_1*Weight1 + Red_2*Weight2 ) Div (Height+Width);
          Green_Mix := ( Green_1*Weight1 + Green_2*Weight2 ) Div (Height+Width);
          Blue_Mix := ( blue_1*Weight1 + Blue_2*Weight2 ) Div (Height+Width);
          MixColor := RGB( Red_Mix, Green_Mix, Blue_Mix );
          Pen.Color := MixColor;
          MoveTo( Center.X, Center.Y );
          LineTo( ARect.Right, ARect.Top + I );

          Weight1 := I;
          Weight2 := Width + Height - I;
          Red_Mix := ( Red_1*Weight1 + Red_2*Weight2 ) Div (Height+Width);
          Green_Mix := ( Green_1*Weight1 + Green_2*Weight2 ) Div (Height+Width);
          Blue_Mix := ( blue_1*Weight1 + Blue_2*Weight2 ) Div (Height+Width);
          MixColor := RGB( Red_Mix, Green_Mix, Blue_Mix );
          Pen.Color := MixColor;
          MoveTo( Center.X, Center.Y );
          LineTo( ARect.Left, ARect.Top + I );

          Inc( I );
        End;
    End;
End;

{========    ===========}

Procedure GradientRect( ACanvas : TCanvas; ARect, ClipRect : TRect; Color1, Color2 : TColor;
                        GradType : TGradType; Params : Byte );
Var
  BRect : TRect;
  CRect : TRect;
Begin
  BRect := ARect;
  CRect := ClipRect;
  CorrectRect( BRect );
  CorrectRect( CRect );
  Case GradType Of
  gtLinear       : LinearGradient( ACanvas, BRect, CRect, Color1, Color2, Params );
  gtLinearDouble : LinearDoubleGradient( ACanvas, BRect, CRect, Color1, Color2, Params );
  gtSquare       : RoundGradient( ACanvas, BRect, CRect, Color2, Color1, Params, True );
  gtSquareDouble : RoundDoubleGradient( ACanvas, BRect, CRect, Color2, Color1, Params, True );
  gtRadial       : RoundGradient( ACanvas, BRect, CRect, Color2, Color1, Params, False );
  gtRadialDouble : RoundDoubleGradient( ACanvas, BRect, CRect, Color2, Color1, Params, False );
  gtConic        : ConicGradient( ACanvas, BRect, CRect, Color1, Color2, Params );
  Else Exit;
  End;{ Case }
End;

{=====    ======}


end.
