unit Paper;

interface

Uses Classes, Windows, Controls, Graphics, Forms, Drawing, Dialogs, Printers,
     SysUtils, JPEG;


Const
  AllPages = -1; //

  MMPerDot = 25.4/72; //     

  MM_Per_Inch  = 25.4;    //      

  Inch_Per_MM = 1 / 25.4;  //       


Type
  TExBrushStyle = ( exSolid, exClear, exHorizontal, exVertical, exFDiagonal,
                    exBDiagonal, exCross, exDiagCross, exLinear, exLinearDouble,
                    exRadial, exRadialDouble, exSquare, exSquareDouble, exConic );

  TUnits = ( unMillimeters, unTenthMillimeters, unCentimeters, unInches, unTenthInches, unHundredthInches );

  TRWMethod = ( PAGE_ADD, PAGE_NEW );

  TImageFormat = ( ifBitmap, ifJPEG, ifMetaFile );
  
Const
  GradientBrushesSet = [ exLinear, exLinearDouble,
                         exRadial, exRadialDouble,
                         exSquare, exSquareDouble,
                         exConic ]; //  
  { commands }
  cmSetDefault = 1;
  cmSetFont = 2;
  cmSetBrush = 3;
  cmSetPen = 4;
  cmDraw = 5;
  cmStretchDraw = 6;
  cmArc = 7;
  cmChord = 8;
  cmPie = 9;
  cmPolygon = 10;
  cmPolyline = 11;
  cmPolyBezier = 12;
  cmEllipse = 13;
  cmRectangle = 14;
  cmRoundRect = 15;
  cmFrameRect = 16;
  cmFillRect = 17;
  cmDrawFocusRect = 18;
  cmTextOut = 19;
  cmTextRect = 20;
  cmLine = 21;
  cmRegion = 22;

Type
  TPixelsPerMM = Record
                   X, Y : Real;
                 End;

  TQDrawItem = Record
                 MemPosition : LongInt;
                 MemSize     : LongInt;
                 ID_Index    : Integer;
               End;

  TStruct = Record
              CommandType  : Word;
              PageNumber   : SmallInt;
              DataPos      : Integer;
              HeaderSize   : Byte;
              DataSize     : Integer;
            End;

  PPointsArray = ^TPointsArray;
  TPointsArray = Array[1..5] Of TPoint;


Type
  TPaperFormatType = Record
                       Name        : String;
                       Index       : SmallInt;
                       Width_MM    : Integer;
                       Height_MM   : Integer;
                       Width_Inch  : Real;
                       Height_Inch : Real;
                     End;

  TPaperFormat = (
                    pfLETTER,
                    pfTABLOID,
                    pfLEDGER,
                    pfLEGAL,
                    pfSTATEMENT,
                    pfEXECUTIVE,
                    pfA3,
                    pfA4,
                    pfA5,
                    pfB4,
                    pfB5,
                    pfFOLIO,
                    pfQUARTO,
                    pf10X14,
                    pf11X17,
                    pfNOTE,
                    pfENV_9,
                    pfENV_10,
                    pfENV_11,
                    pfENV_12,
                    pfENV_14,
                    pfCSHEET,
                    pfDSHEET,
                    pfESHEET,
                    pfENV_DL,
                    pfENV_C5,
                    pfENV_C3,
                    pfENV_C4,
                    pfENV_C6,
                    pfENV_C65,
                    pfENV_B4,
                    pfENV_B5,
                    pfENV_B6,
                    pfENV_ITALY,
                    pfENV_MONARCH,
                    pfENV_PERSONAL,
                    pfFANFOLD_US,
                    pfFANFOLD_STD_GERMAN,
                    pfFANFOLD_LGL_GERMAN,
                    pfISO_B4,
                    pfJAPANESE_POSTCARD,
                    pf9X11,
                    pf10X11,
                    pf15X11,
                    pfENV_INVITE,
                    pfLETTER_EXTRA,
                    pfLEGAL_EXTRA,
                    pfTABLOID_EXTRA,
                    pfA4_EXTRA,
                    pfLETTER_TRANSVERSE,
                    pfA4_TRANSVERSE,
                    pfLETTER_EXTRA_TRANSVERSE,
                    pfA_PLUS,
                    pfB_PLUS,
                    pfLETTER_PLUS,
                    pfA4_PLUS,
                    pfA5_TRANSVERSE,
                    pfB5_TRANSVERSE,
                    pfA3_EXTRA,
                    pfA5_EXTRA,
                    pfB5_EXTRA,
                    pfA2,
                    pfA3_TRANSVERSE,
                    pfA3_EXTRA_TRANSVERSE,
                    pfDBL_JAPANESE_POSTCARD,
                    pfA6,
                    pfP16K,
                    pfP32K,
                    pfPENV_1,
                    pfPENV_2,
                    pfPENV_3,
                    pfPENV_4,
                    pfPENV_5,
                    pfPENV_6,
                    pfPENV_7,
                    pfPENV_8,
                    pfPENV_9,
                    pfPENV_10,
                    pfCustom
   );

Const
  PaperFormats : Array [ TPaperFormat ] Of TPaperFormatType = (

  ( Name        : 'LETTER';
    Index       : DMPAPER_LETTER;
    Width_MM    : Round(8.5*MM_Per_Inch);
    Height_MM   : Round(11*MM_Per_Inch);
    Width_Inch  : 8.5;
    Height_Inch : 11 ),

  ( Name        : 'TABLOID';
    Index       : 3;
    Width_MM    : Round(11*MM_Per_Inch);
    Height_MM   : Round(7*MM_Per_Inch);
    Width_Inch  : 11;
    Height_Inch : 7 ),

  ( Name        : 'LEDGER';
    Index       : 4;
    Width_MM    : Round(17*MM_Per_Inch);
    Height_MM   : Round(11*MM_Per_Inch);
    Width_Inch  : 17;
    Height_Inch : 11 ),

  ( Name        : 'LEGAL';
    Index       : 5;
    Width_MM    : Round(8.5*MM_Per_Inch);
    Height_MM   : Round(14*MM_Per_Inch);
    Width_Inch  : 8.5;
    Height_Inch : 14 ),

  ( Name        : 'STATEMENT';
    Index       : 6;
    Width_MM    : Round(5.5*MM_Per_Inch);
    Height_MM   : Round(8.5*MM_Per_Inch);
    Width_Inch  : 5.5;
    Height_Inch : 8.5 ),

  ( Name        : 'EXECUTIVE';
    Index       : 7;
    Width_MM    : Round( 7.25*MM_Per_Inch);
    Height_MM   : Round( 10.5*MM_Per_Inch);
    Width_Inch  : 7.25;
    Height_Inch : 10.5),

  ( Name        : 'A3';
    Index       : 8;
    Width_MM    : 297;
    Height_MM   : 420;
    Width_Inch  : 297*Inch_Per_MM;
    Height_Inch : 420*Inch_Per_MM ),

  ( Name        : 'A4';
    Index       : DMPAPER_A4;
    Width_MM    : 210;
    Height_MM   : 297;
    Width_Inch  : 210*Inch_Per_MM;
    Height_Inch : 297*Inch_Per_MM ),

  ( Name        : 'A5';
    Index       : 11;
    Width_MM    : 148;
    Height_MM   : 210;
    Width_Inch  : 148*Inch_Per_MM;
    Height_Inch : 210*Inch_Per_MM ),

  ( Name        : 'B4';
    Index       : 12;
    Width_MM    : 250;
    Height_MM   : 354;
    Width_Inch  : 250*Inch_Per_MM;
    Height_Inch : 324*Inch_Per_MM ),

  ( Name        : 'B5';
    Index       : 13;
    Width_MM    : 182;
    Height_MM   : 257;
    Width_Inch  : 182*Inch_Per_MM;
    Height_Inch : 257*Inch_Per_MM ),

  ( Name        : 'Folio';
    Index       : 14;
    Width_MM    : Round( 8.5*MM_Per_Inch);
    Height_MM   : Round( 13*MM_Per_Inch);
    Width_Inch  : 8.5;
    Height_Inch : 13 ),

  ( Name        : 'QUARTO';
    Index       : 15;
    Width_MM    : 215;
    Height_MM   : 275;
    Width_Inch  : 215*Inch_Per_MM;
    Height_Inch : 275*Inch_Per_MM ),

  ( Name        : '10x14';
    Index       : 16;
    Width_MM    : Round(10*MM_Per_Inch);
    Height_MM   : Round(14*MM_Per_Inch);
    Width_Inch  : 10;
    Height_Inch : 14 ),

  ( Name        : '11x17';
    Index       : 17;
    Width_MM    : Round(11*MM_Per_Inch);
    Height_MM   : Round(17*MM_Per_Inch);
    Width_Inch  : 11;
    Height_Inch : 17 ),

  ( Name        : 'Note';
    Index       : 18;
    Width_MM    : Round(8.5*MM_Per_Inch);
    Height_MM   : Round(11*MM_Per_Inch);
    Width_Inch  : 8.5;
    Height_Inch : 11 ),

  ( Name        : 'Envelope #9';
    Index       : 19;
    Width_MM    : Round(3.875*MM_Per_Inch);
    Height_MM   : Round(8.875*MM_Per_Inch);
    Width_Inch  : 3.875;
    Height_Inch : 8.875 ),

  ( Name        : 'Envelope #10';
    Index       : 20;
    Width_MM    : Round(4.125*MM_Per_Inch);
    Height_MM   : Round(9.5*MM_Per_Inch);
    Width_Inch  : 4.125;
    Height_Inch : 9.5 ),

  ( Name        : 'Envelope #11';
    Index       : 21;
    Width_MM    : Round(4.5*MM_Per_Inch);
    Height_MM   : Round(10.375*MM_Per_Inch);
    Width_Inch  : 4.5;
    Height_Inch : 10.375 ),

  ( Name        : 'Envelope #12';
    Index       : 22;
    Width_MM    : Round(4.75*MM_Per_Inch);
    Height_MM   : Round(11*MM_Per_Inch);
    Width_Inch  : 4.75;
    Height_Inch : 11 ),

  ( Name        : 'Envelope #14';
    Index       : 23;
    Width_MM    : Round(5*MM_Per_Inch);
    Height_MM   : Round(11.5*MM_Per_Inch);
    Width_Inch  : 5;
    Height_Inch : 11.5 ),

  ( Name        : 'C sheet';
    Index       : 24;
    Width_MM    : Round(17*MM_Per_Inch);
    Height_MM   : Round(22*MM_Per_Inch);
    Width_Inch  : 17;
    Height_Inch : 22 ),

  ( Name        : 'D sheet';
    Index       : 25;
    Width_MM    : Round(22*MM_Per_Inch);
    Height_MM   : Round(34*MM_Per_Inch);
    Width_Inch  : 22;
    Height_Inch : 34 ),

  ( Name        : 'E sheet';
    Index       : 26;
    Width_MM    : Round(34*MM_Per_Inch);
    Height_MM   : Round(44*MM_Per_Inch);
    Width_Inch  : 34;
    Height_Inch : 44 ),

  ( Name        : 'DL Envelope';
    Index       : 27;
    Width_MM    : 110;
    Height_MM   : 220;
    Width_Inch  : 110*Inch_Per_MM;
    Height_Inch : 220*Inch_Per_MM ),

  ( Name        : 'C5 Envelope';
    Index       : 28;
    Width_MM    : 162;
    Height_MM   : 229;
    Width_Inch  : 110*Inch_Per_MM;
    Height_Inch : 220*Inch_Per_MM ),

  ( Name        : 'C3 Envelope';
    Index       : 29;
    Width_MM    : 324;
    Height_MM   : 458;
    Width_Inch  : 324*Inch_Per_MM;
    Height_Inch : 458*Inch_Per_MM ),

  ( Name        : 'C4 Envelope';
    Index       : 30;
    Width_MM    : 229;
    Height_MM   : 324;
    Width_Inch  : 229*Inch_Per_MM;
    Height_Inch : 324*Inch_Per_MM ),

  ( Name        : 'C6 Envelope';
    Index       : 31;
    Width_MM    : 114;
    Height_MM   : 162;
    Width_Inch  : 114*Inch_Per_MM;
    Height_Inch : 162*Inch_Per_MM ),

  ( Name        : 'C65 Envelope';
    Index       : 32;
    Width_MM    : 114;
    Height_MM   : 229;
    Width_Inch  : 114*Inch_Per_MM;
    Height_Inch : 220*Inch_Per_MM ),

  ( Name        : 'B4 Envelope';
    Index       : 33;
    Width_MM    : 250;
    Height_MM   : 353;
    Width_Inch  : 250*Inch_Per_MM;
    Height_Inch : 353*Inch_Per_MM ),

  ( Name        : 'B5 Envelope';
    Index       : 34;
    Width_MM    : 176;
    Height_MM   : 250;
    Width_Inch  : 176*Inch_Per_MM;
    Height_Inch : 250*Inch_Per_MM ),

  ( Name        : 'B6 Envelope';
    Index       : 35;
    Width_MM    : 176;
    Height_MM   : 125;
    Width_Inch  : 176*Inch_Per_MM;
    Height_Inch : 125*Inch_Per_MM ),

  ( Name        : 'Italy Envelope';
    Index       : 36;
    Width_MM    : 110;
    Height_MM   : 230;
    Width_Inch  : 110*Inch_Per_MM;
    Height_Inch : 230*Inch_Per_MM ),

  ( Name        : 'Monarch Envelope';
    Index       : 37;
    Width_MM    : Round(3.875*MM_Per_Inch);
    Height_MM   : Round(7.5*MM_Per_Inch);
    Width_Inch  : 3.875;
    Height_Inch : 7.5 ),

  ( Name        : '6 3/4 Envelope';
    Index       : 38;
    Width_MM    : Round(3.625*MM_Per_Inch);
    Height_MM   : Round(5.5*MM_Per_Inch);
    Width_Inch  : 3.625;
    Height_Inch : 6.5 ),

  ( Name        : 'US Std Fanfold';
    Index       : 39;
    Width_MM    : Round(4.875*MM_Per_Inch);
    Height_MM   : Round(11*MM_Per_Inch);
    Width_Inch  : 4.875;
    Height_Inch : 11 ),

  ( Name        : 'German Std Fanfold';
    Index       : 40;
    Width_MM    : Round(8.5*MM_Per_Inch);
    Height_MM   : Round(12*MM_Per_Inch);
    Width_Inch  : 8.5*Inch_Per_MM;
    Height_Inch : 12*Inch_Per_MM ),

  ( Name        : 'German Legal Fanfold';
    Index       : 41;
    Width_MM    : Round(8.5*MM_Per_Inch);
    Height_MM   : Round(13*MM_Per_Inch);
    Width_Inch  : 8.5;
    Height_Inch : 13 ),

  ( Name        : 'B4 (ISO)';
    Index       : 42;
    Width_MM    : 250;
    Height_MM   : 353;
    Width_Inch  : 250*Inch_Per_MM;
    Height_Inch : 353*Inch_Per_MM ),

  ( Name        : 'Japanese Postcard';
    Index       : 43;
    Width_MM    : 100;
    Height_MM   : 148;
    Width_Inch  : 100*Inch_Per_MM;
    Height_Inch : 148*Inch_Per_MM ),

  ( Name        : '9x11';
    Index       : 44;
    Width_MM    : Round(9*MM_Per_Inch);
    Height_MM   : Round(11*MM_Per_Inch);
    Width_Inch  : 9;
    Height_Inch : 11 ),

  ( Name        : '10x11';
    Index       : 45;
    Width_MM    : Round(10*MM_Per_Inch);
    Height_MM   : Round(11*MM_Per_Inch);
    Width_Inch  : 10;
    Height_Inch : 11 ),

  ( Name        : '15x11';
    Index       : 46;
    Width_MM    : Round(15*MM_Per_Inch);
    Height_MM   : Round(11*MM_Per_Inch);
    Width_Inch  : 15;
    Height_Inch : 11 ),

  ( Name        : 'Invite Envelope';
    Index       : 47;
    Width_MM    : 220;
    Height_MM   : 220;
    Width_Inch  : 220*Inch_Per_MM;
    Height_Inch : 220*Inch_Per_MM ),

  ( Name        : 'Letter Extra';
    Index       : 50;
    Width_MM    : Round(9.275*MM_Per_Inch);
    Height_MM   : Round(12*MM_Per_Inch);
    Width_Inch  : 9.275;
    Height_Inch : 12 ),

  ( Name        : 'Legal Extra';
    Index       : 51;
    Width_MM    : Round(9.275*MM_Per_Inch);
    Height_MM   : Round(15*MM_Per_Inch);
    Width_Inch  : 9.275;
    Height_Inch : 15 ),

  ( Name        : 'Tabloid Extra';
    Index       : 52;
    Width_MM    : Round(11.69*MM_Per_Inch);
    Height_MM   : Round(18*MM_Per_Inch);
    Width_Inch  : 11.69;
    Height_Inch : 18 ),

  ( Name        : 'A4 Extra';
    Index       : 53;
    Width_MM    : Round(9.27*MM_Per_Inch);
    Height_MM   : Round(12.69*MM_Per_Inch);
    Width_Inch  : 9.27;
    Height_Inch : 12.69 ),

  ( Name        : 'Letter Transverse';
    Index       : 54;
    Width_MM    : Round(8.275*MM_Per_Inch);
    Height_MM   : Round(11*MM_Per_Inch);
    Width_Inch  : 8.275;
    Height_Inch : 11 ),

  ( Name        : 'A4 Transverse';
    Index       : 55;
    Width_MM    : 210;
    Height_MM   : 297;
    Width_Inch  : 210*Inch_Per_MM;
    Height_Inch : 297*Inch_Per_MM ),

  ( Name        : 'Letter Extra Transverse';
    Index       : 56;
    Width_MM    : Round(9.275*MM_Per_Inch);
    Height_MM   : Round(12*MM_Per_Inch);
    Width_Inch  : 9.275;
    Height_Inch : 12 ),

  ( Name        : 'SuperA SuperA A4';
    Index       : 57;
    Width_MM    : 227;
    Height_MM   : 356;
    Width_Inch  : 227*Inch_Per_MM;
    Height_Inch : 356*Inch_Per_MM ),

  ( Name        : 'SuperB SuperB A3';
    Index       : 58;
    Width_MM    : 305;
    Height_MM   : 487;
    Width_Inch  : 305*Inch_Per_MM;
    Height_Inch : 487*Inch_Per_MM ),

  ( Name        : 'Letter Plus';
    Index       : 59;
    Width_MM    : Round(8.5*MM_Per_Inch);
    Height_MM   : Round(12.69*MM_Per_Inch);
    Width_Inch  : 8.5;
    Height_Inch : 12.69 ),

  ( Name        : 'A4 Plus';
    Index       : 60;
    Width_MM    : 210;
    Height_MM   : 330;
    Width_Inch  : 210*Inch_Per_MM;
    Height_Inch : 330*Inch_Per_MM ),

  ( Name        : 'A5 Transverse';
    Index       : 61;
    Width_MM    : 148;
    Height_MM   : 210;
    Width_Inch  : 148*Inch_Per_MM;
    Height_Inch : 210*Inch_Per_MM ),

  ( Name        : 'B5 (JIS) Transverse';
    Index       : 62;
    Width_MM    : 182;
    Height_MM   : 257;
    Width_Inch  : 182*Inch_Per_MM;
    Height_Inch : 257*Inch_Per_MM ),

  ( Name        : 'A3 Extra';
    Index       : 63;
    Width_MM    : 322;
    Height_MM   : 445;
    Width_Inch  : 322*Inch_Per_MM;
    Height_Inch : 445*Inch_Per_MM ),

  ( Name        : 'A5 Extra';
    Index       : 64;
    Width_MM    : 174;
    Height_MM   : 235;
    Width_Inch  : 174*Inch_Per_MM;
    Height_Inch : 235*Inch_Per_MM ),

  ( Name        : 'B5 (ISO) Extra';
    Index       : 65;
    Width_MM    : 201;
    Height_MM   : 276;
    Width_Inch  : 201*Inch_Per_MM;
    Height_Inch : 276*Inch_Per_MM ),

  ( Name        : 'A2';
    Index       : 66;
    Width_MM    : 420;
    Height_MM   : 594;
    Width_Inch  : 420*Inch_Per_MM;
    Height_Inch : 594*Inch_Per_MM ),

  ( Name        : 'A3 Transverse';
    Index       : 67;
    Width_MM    : 297;
    Height_MM   : 420;
    Width_Inch  : 297*Inch_Per_MM;
    Height_Inch : 420*Inch_Per_MM ),

  ( Name        : 'A3 Extra Transverse';
    Index       : 68;
    Width_MM    : 322;
    Height_MM   : 445;
    Width_Inch  : 322*Inch_Per_MM;
    Height_Inch : 445*Inch_Per_MM ),
{
 ** the following sizes are reserved for the Far East version of Win95.
 ** Rotated papers rotate the physical page but not the logical page.
}
  ( Name        : 'Japanese Double Postcard';
    Index       : 69;
    Width_MM    : 200;
    Height_MM   : 148;
    Width_Inch  : 200*Inch_Per_MM;
    Height_Inch : 148*Inch_Per_MM ),

  ( Name        : 'A6';
    Index       : 70;
    Width_MM    : 105;
    Height_MM   : 148;
    Width_Inch  : 105*Inch_Per_MM;
    Height_Inch : 148*Inch_Per_MM ),

  ( Name        : 'PRC 16K';
    Index       : 93;
    Width_MM    : 146;
    Height_MM   : 215;
    Width_Inch  : 146*Inch_Per_MM;
    Height_Inch : 215*Inch_Per_MM ),

  ( Name        : 'PRC 32K';
    Index       : 94;
    Width_MM    : 97;
    Height_MM   : 151;
    Width_Inch  : 97*Inch_Per_MM;
    Height_Inch : 151*Inch_Per_MM ),

  ( Name        : 'PRC Envelope #1';
    Index       : 96;
    Width_MM    : 102;
    Height_MM   : 165;
    Width_Inch  : 102*Inch_Per_MM;
    Height_Inch : 165*Inch_Per_MM ),

  ( Name        : 'PRC Envelope #2';
    Index       : 97;
    Width_MM    : 102;
    Height_MM   : 176;
    Width_Inch  : 102*Inch_Per_MM;
    Height_Inch : 176*Inch_Per_MM ),

  ( Name        : 'PRC Envelope #3';
    Index       : 98;
    Width_MM    : 125;
    Height_MM   : 176;
    Width_Inch  : 125*Inch_Per_MM;
    Height_Inch : 176*Inch_Per_MM ),

  ( Name        : 'PRC Envelope #4';
    Index       : 99;
    Width_MM    : 110;
    Height_MM   : 208;
    Width_Inch  : 110*Inch_Per_MM;
    Height_Inch : 208*Inch_Per_MM ),

  ( Name        : 'PRC Envelope #5';
    Index       : 100;
    Width_MM    : 110;
    Height_MM   : 220;
    Width_Inch  : 110*Inch_Per_MM;
    Height_Inch : 220*Inch_Per_MM ),

  ( Name        : 'PRC Envelope #6';
    Index       : 101;
    Width_MM    : 120;
    Height_MM   : 230;
    Width_Inch  : 120*Inch_Per_MM;
    Height_Inch : 230*Inch_Per_MM ),

  ( Name        : 'PRC Envelope #7';
    Index       : 102;
    Width_MM    : 160;
    Height_MM   : 230;
    Width_Inch  : 160*Inch_Per_MM;
    Height_Inch : 230*Inch_Per_MM ),

  ( Name        : 'PRC Envelope #8';
    Index       : 103;
    Width_MM    : 120;
    Height_MM   : 309;
    Width_Inch  : 120*Inch_Per_MM;
    Height_Inch : 309*Inch_Per_MM ),

  ( Name        : 'PRC Envelope #9';
    Index       : 104;
    Width_MM    : 229;
    Height_MM   : 324;
    Width_Inch  : 229*Inch_Per_MM;
    Height_Inch : 324*Inch_Per_MM ),

  ( Name        : 'PRC Envelope #10';
    Index       : 105;
    Width_MM    : 324;
    Height_MM   : 458;
    Width_Inch  : 324*Inch_Per_MM;
    Height_Inch : 458*Inch_Per_MM ),

  ( Name        : 'Custom';
    Index       : 256;
    Width_MM    : 100;
    Height_MM   : 100;
    Width_Inch  : 100*Inch_Per_MM;
    Height_Inch : 100*Inch_Per_MM; ) );


{ TPrinterSetup }

Type
 TPrinterSetup = class( TPersistent )
  private
    { Private declarations }
    FOrientation : TPrinterOrientation;
    FPaperFormat : TPaperFormat;
    FWidth_MM    : Integer;
    FHeight_MM   : Integer;
    FWidth_Inch  : Real;
    FHeight_Inch : Real;
    FPMMX        : Real;
    FPMMY        : Real;
    FOnChangeFormat : TNotifyEvent;
    Procedure SetOrientation( Value : TPrinterOrientation );
    Procedure SetPaperFormat( Value : TPaperFormat );
    Procedure SetWidth_MM( Value : Integer );
    Procedure SetHeight_MM( Value : Integer );
    Function IsCustomType : Boolean;
    Function GetPaperFormatName : String;
    Procedure SetPMM( Value : Real );
  protected
    { Protected declarations }
  public
    { Public declarations }
    Constructor Create;
    Destructor Destroy; override;
    Procedure GetPaperSize( Format : TPaperFormat; Var Width_Inch, Height_Inch : Real; Var Width_MM, Height_MM : Integer );
    Function GetFormatName( Format : TPaperFormat ) : String;
    Function SetFormatByName( FormatName : String ) : Boolean;
  published
    { published declarations }
    Property Orientation      : TPrinterOrientation Read FOrientation Write SetOrientation Default poPortrait;
    Property PaperFormat      : TPaperFormat Read FPaperFormat Write SetPaperFormat;
    Property PaperFormatName  : String Read GetPaperFormatName;
    Property PaperWidth_MM    : Integer Read FWidth_MM Write SetWidth_MM Stored IsCustomType;
    Property PaperHeight_MM   : Integer Read FHeight_MM Write SetHeight_MM Stored IsCustomType;
    Property PaperWidth_Inch  : Real Read FWidth_Inch;
    Property PaperHeight_Inch : Real Read FHeight_Inch;
    Property PixelsPerMM_byX  : Real Read FPMMX Write SetPMM;
    Property PixelsPerMM_byY  : Real Read FPMMY Write SetPMM;
    Property OnChangeFormat   : TNotifyEvent Read FOnChangeFormat Write FOnChangeFormat;
  End;

{ TIndentions }

Type
  TIndentions = class(TPersistent)
  private
    { Private declarations }
    FVisible : Boolean;
    FMirror  : Boolean;
    FLeft    : Integer;
    FRight   : Integer;
    FTop     : Integer;
    FBottom  : Integer;
    Procedure SetVisible( Value : Boolean );
    Procedure SetMirror( Value : Boolean );
    Procedure SetLeft( Value : Integer );
    Procedure SetRight( Value : Integer );
    Procedure SetTop( Value : Integer );
    Procedure SetBottom( Value : Integer );
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create;
    destructor Destroy; override;
//    Procedure Assign( Sourse : TIndentions );
  published
    Property Visible : Boolean Read FVisible Write SetVisible Default True;
    Property Mirror  : Boolean Read FMirror Write SetMirror Default False;
    Property Left    : Integer Read FLeft Write SetLeft Default 10;
    Property Right   : Integer Read FRight Write SetRight Default 10;
    Property Top     : Integer Read FTop Write SetTop Default 10;
    Property Bottom  : Integer Read FBottom Write SetBottom Default 10;
  End;

Type
  TNotifyDrawEvent = procedure( Sender : TObject; ItemType : Word; Index : Integer ) Of Object;

Type
  TPaper = class(TPersistent)
  private
    FWidth        : Integer; //    
    FHeight       : Integer; //    
    FPixelsPerMM  : TPixelsPerMM; //    
    FUnits        : TUnits; // 
    FIndentions   : TIndentions;
    FPrinterSetup : TPrinterSetup;
    FOnDrawItem   : TNotifyDrawEvent;
    FChangeSize   : TNotifyEvent;
    FChangeUnits  : TNotifyEvent;
    Procedure WriteString( Mem : TMemoryStream; S : String; Size : Integer );
    Procedure ReadString( Mem : TMemoryStream; Var S : String; Size : Integer );
    Procedure SetUnits( Value : TUnits );
    Procedure SetIndentions( Value : TIndentions );
    Procedure SetPrinterSetup( Value : TPrinterSetup );
    Procedure OnChangePrinterFormat( Sender : TObject );
  protected
    MemoryPaper  : TMemoryStream;
    SizesList    : TList;
    ActivePage   : SmallInt;
    Procedure SetDefault( ACanvas : TCanvas; Zoom, Scale : Real );
    Procedure ReCountDataPos;
  public
    { Public declarations }
    CompressionQuality : TJPEGQualityRange;
    constructor Create;
    destructor Destroy; override;
    Procedure SetFont( Name : String; Size : Integer; Style : TFontStyles; Color : TColor );
    Procedure SetBrush( Style: TExBrushStyle; Color, Color2 : TColor; Bitmap : TBitmap; Params : Byte );
    Procedure SetPen( Style : TExBrushStyle; Color : TColor; Width : Real );
    procedure BrushCopy( const Dest : TRect; Bitmap : TBitmap;
                         const Source : TRect; Color : TColor );
    procedure Draw( X, Y : Integer; Graphic : TGraphic );
    procedure StretchDraw( const Rect : TRect; Graphic : TGraphic );
    procedure Arc( X1, Y1, X2, Y2, X3, Y3, X4, Y4 : Integer );
    procedure Chord( X1, Y1, X2, Y2, X3, Y3, X4, Y4 : Integer );
    procedure Pie( X1, Y1, X2, Y2, X3, Y3, X4, Y4 : Integer );
    procedure Polygon( const Points : array of TPoint );
    procedure Polyline( const Points : array of TPoint );
    procedure PolyBezier( const Points : array of TPoint );
    procedure Ellipse( X1, Y1, X2, Y2 : Integer ); overload;
    procedure Ellipse( const Rect : TRect ); overload;
    procedure Rectangle( X1, Y1, X2, Y2 : Integer ); overload;
    procedure Rectangle( const Rect : TRect ); overload;
    procedure DrawFocusRect( const Rect : TRect );
    procedure FillRect( const Rect : TRect );
    procedure FrameRect( const Rect : TRect );
    procedure RoundRect( X1, Y1, X2, Y2, X3, Y3 : Integer );
    procedure Line( X1, Y1, X2, Y2 : Integer );
    procedure TextOut( X, Y : Integer; const Text: string);
    procedure TextRect( Rect: TRect; X, Y : Integer; const Text: string);
    function TextExtent( const Text : string ) : TSize;
    function TextWidth( const Text : string ) : Integer;
    function TextHeight( const Text : string ) : Integer; 
    Procedure DrawRegion( RGN : HRGN );
    {********}
    Procedure DrawToCanvas( ACanvas : TCanvas; Zoom, Scale : Real; Page : SmallInt );
    Procedure SetPage( Page : SmallInt );
    Function GetPage : SmallInt;
    Function GetItemsCount : Integer;
    Function GetItemsCountOnPage( PageNumber : SmallInt ) : Integer;
    Procedure Clear;
    Procedure SaveToFile( FileName : String; WriteMethod : TRWMethod );
    Procedure LoadFromFile( FileName : String; ReadMethod : TRWMethod );
    Procedure SavePageToFile( FileName : String; Page : SmallInt; WriteMethod : TRWMethod );
    Procedure LoadPageFromFile( FileName : String; Page : SmallInt; ReadMethod : TRWMethod );
    Procedure ExportPageToFile( FileName : String; PageNumber : SmallInt; ImageFormat : TImageFormat );
  published
    { Published declarations }
    Property Width        : Integer Read FWidth;
    Property Height       : Integer Read FHeight;
    Property PixelsPerMM  : TPixelsPerMM Read FPixelsPerMM;
    Property Units        : TUnits Read FUnits Write SetUnits;
    Property Indentions   : TIndentions Read FIndentions Write SetIndentions;
    Property PrinterSetup : TPrinterSetup Read FPrinterSetup Write SetPrinterSetup;
    { events }
    Property OnChangeSize  : TNotifyEvent Read FChangeSize Write FChangeSize;
    Property OnDrawObject  : TNotifyDrawEvent Read FOnDrawItem Write FOnDrawItem;
    Property OnChangeUnits : TNotifyEvent Read FChangeUnits Write FChangeUnits;
  end;

implementation

Type
  TDraw = Record
            X, Y   : Integer;
            Width  : Integer;
            Height : Integer;
            GrType : Byte;
            Transp : Boolean;
            Color  : TColor;
          End;
  TDrawFocusRect = Record
                     Rect : TRect;
                   End;
  TPie = Record
           X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer;
         End;
  TPoly = Record
            PointN : Integer;
            Points : Array Of TPoint;
          End;
  TRectangle = Record
                 X1, Y1, X2, Y2 : Integer;
               End;
  TRoundRect = Record
                 X1, Y1, X2, Y2, X3, Y3 : Integer;
               End;
  TStretchDraw = Record
                   Rect   : TRect;
                   GrType : Byte;
                   Transp : Boolean;
                   Color  : TColor;
                 End;
  TTextOut = Record
               X, Y : Integer;
             End;
  TTextRect = Record
               Rect : TRect;
               X, Y : Integer;
              End;
  TFontRec = Record
               Name   : String[32];
               Size   : Integer;
               Color  : TColor;
               Style  : Byte;
             End;
  TBrushRec = Record
                Style  : Byte;
                Color  : TColor;
                Color2 : TColor;
                Params : Byte;
             End;
  TPenRec = Record
              Style : TExBrushStyle;
              Color : TColor;
              Width : Real;
            End;

Var
  varDraw        : TDraw;
  varPie         : TPie;
  varPoly        : TPoly;
  varRectangle   : TRectangle;
  varRoundRect   : TRoundRect;
  varStretchDraw : TStretchDraw;
  varTextOut     : TTextOut;
  varTextRect    : TTextRect;
  varFont        : TFontRec;
  varBrush       : TBrushRec;
  varPen         : TPenRec;

Type
  TFileHeader = Record
                  PreviewMark : Array[1..3] Of Char;
                  Version     : Word;
                  ItemsCount  : Integer;
                  DataPos     : Integer;
                  PaperFormat : TPaperFormat;
                  PaperWidth  : Integer;
                  PaperHeight : Integer;
                  Units       : TUnits;
                  Orientation : TPrinterOrientation; 
                End;

Const
  MaxPointsNumber = 10000;

Var
  Rgn_Points : Array[1..MaxPointsNumber] Of TPoint;

Var
  CurrUnits : Real;

Procedure WriteToLog( Msg : String );
Var
  F : TEXT;
Begin
  Assign( F, 'ErrorLog.txt' );
  Append( F );
  If FileExists('ErrorLog.txt')
    Then Begin
           Try
             Append( F );
           Except
             Exit;
           End
         End
    Else Begin
           Try
             ReWrite( F );
           Except
             Exit;
           End;
         End;
  WriteLn( F, DateTimeToStr( Now ) + ' : ' + Msg );
  CloseFile( F );
End;

{ ========= TIndentions =========== }

Constructor TIndentions.Create;
Begin
  Inherited Create;
  FVisible := True;
  FMirror := False;
  FLeft := 10;
  FTop := 10;
  FRight := 10;
  FBottom := 10;
End;

Destructor TIndentions.Destroy;
Begin
  Inherited Destroy;
End;

Procedure TIndentions.SetMirror( Value : Boolean );
Begin
  FMirror := Value;
  FRight := Left;
  FBottom := Top;
End;

Procedure TIndentions.SetVisible( Value : Boolean );
Begin
  If FVisible <> Value
    Then FVisible := Value;
End;

Procedure TIndentions.SetLeft( Value : Integer );
Begin
  FLeft := Value;
  If Mirror
    Then FRight := Value;
End;

Procedure TIndentions.SetRight( Value : Integer );
Begin
  If Mirror
    Then FRight := Left
    Else FRight := Value;
End;

Procedure TIndentions.SetTop( Value : Integer );
Begin
  FTop := Value;
  If Mirror
    Then FBottom := Value;
End;

Procedure TIndentions.SetBottom( Value : Integer );
Begin
  If Mirror
    Then FBottom := Top
    Else FBottom := Value;
End;

{ ========= TPrinterSetup =========== }

Constructor TPrinterSetup.Create;
Begin
  Inherited;
  FPMMX := 0;
  FPMMY := 0;
  FPaperFormat := pfA4;
  FOrientation := poPortrait;
  Printer.Orientation := poPortrait;
  GetPaperSize( FPaperFormat, FWidth_Inch, FHeight_Inch, FWidth_MM, FHeight_MM );
  SetPaperFormat( pfA4 );
End;

Destructor TPrinterSetup.Destroy;
Begin
  Inherited;
End;

{ Private declarations }

Procedure TPrinterSetup.SetOrientation( Value : TPrinterOrientation );
Var
  WMM, HMM     : Integer;
  WInch, HInch : Real;
Begin
  If Value <> FOrientation
    Then Begin
           FOrientation := Value;
           Printer.Orientation := Value;
           If PaperFormat = pfCustom
             Then Begin
                    GetPaperSize( FPaperFormat, WInch, HInch, WMM, HMM  );
                    FWidth_Inch := HInch;
                    FHeight_Inch := WInch;
                    FWidth_MM := HMM;
                    FHeight_MM := WMM;
                  End
             Else Begin
                    GetPaperSize( FPaperFormat, FWidth_Inch, FHeight_Inch, FWidth_MM, FHeight_MM );
                  End;
           If FWidth_MM <> 0
             Then FPMMX := Printer.PageWidth / FWidth_MM;
           If FHeight_MM <> 0
             Then FPMMY := Printer.PageHeight / FHeight_MM;
         End;
  If Assigned( FOnChangeFormat )
    Then FOnChangeFormat( Self );
End;

Procedure TPrinterSetup.SetPaperFormat( Value : TPaperFormat );
Begin
  FPaperFormat := Value;
  If FPaperFormat <> pfCustom
    Then Begin
           GetPaperSize( FPaperFormat, FWidth_Inch, FHeight_Inch, FWidth_MM, FHeight_MM );
           If FWidth_MM <> 0
             Then FPMMX := Printer.PageWidth / FWidth_MM;
           If FHeight_MM <> 0
             Then FPMMY := Printer.PageHeight / FHeight_MM;
         End;
  If Assigned( FOnChangeFormat )
    Then FOnChangeFormat( Self );
End;

Procedure TPrinterSetup.SetWidth_MM( Value : Integer );
Begin
  If FWidth_MM <> Value
    Then Begin
           FPaperFormat := pfCustom;
           FWidth_MM := Value;
           FWidth_Inch := Inch_Per_MM * Value;
           If FWidth_MM <> 0
             Then FPMMX := Printer.PageWidth / FWidth_MM;
         End;
  If Assigned( FOnChangeFormat )
    Then FOnChangeFormat( Self );
End;

Procedure TPrinterSetup.SetHeight_MM( Value : Integer );
Begin
  If FHeight_MM <> Value
    Then Begin
           FPaperFormat := pfCustom;
           FHeight_MM := Value;
           FHeight_Inch := Inch_Per_MM * Value;
           If FHeight_MM <> 0
             Then FPMMY := Printer.PageHeight / FHeight_MM;
         End;
  If Assigned( FOnChangeFormat )
    Then FOnChangeFormat( Self );
End;

Function TPrinterSetup.IsCustomType : Boolean;
Begin
  If FPaperFormat = pfCustom
    Then Result := True
    Else Result := False;
End;

Function TPrinterSetup.GetPaperFormatName : String;
Begin
  Result := PaperFormats[FPaperFormat].Name;
End;

Procedure TPrinterSetup.SetPMM( Value : Real );
Begin
End;

{ Public declarations }

Procedure TPrinterSetup.GetPaperSize( Format : TPaperFormat; Var Width_Inch, Height_Inch : Real; Var Width_MM, Height_MM : Integer );
Begin
  If PaperFormat <> pfCustom
    Then Begin
           If Orientation = poPortrait
             Then Begin
                    Width_Inch := PaperFormats[FPaperFormat].Width_Inch;
                    Height_Inch := PaperFormats[FPaperFormat].Height_Inch;
                    Width_MM := PaperFormats[FPaperFormat].Width_MM;
                    Height_MM := PaperFormats[FPaperFormat].Height_MM;
                  End
             Else Begin
                    Width_Inch := PaperFormats[FPaperFormat].Height_Inch;
                    Height_Inch := PaperFormats[FPaperFormat].Width_Inch;
                    Width_MM := PaperFormats[FPaperFormat].Height_MM;
                    Height_MM := PaperFormats[FPaperFormat].Width_MM;
                  End;
         End
    Else Begin
           Width_Inch := PaperWidth_Inch;
           Height_Inch := PaperHeight_Inch;
           Width_MM := PaperWidth_MM;
           Height_MM := PaperHeight_MM;
         End;
End;

Function TPrinterSetup.GetFormatName( Format : TPaperFormat ) : String;
Begin
  Result := PaperFormats[Format].Name;
End;

Function TPrinterSetup.SetFormatByName( FormatName : String ) : Boolean;
var
  I : TPaperFormat;
Begin
  Result := False;
  For I := Low( TPaperFormat ) To High( TPaperFormat ) Do
    Begin
      If UpperCase(GetFormatName( I )) = UpperCase(FormatName)
        Then Begin
               PaperFormat := I;
               Result := True;
               Exit;
             End;
    End;
End;


{  ========== TPaper ============ }

{ Private }

Procedure TPaper.WriteString( Mem : TMemoryStream; S : String; Size : Integer );
Var
  I : Integer;
  C : Char;
Begin
  For I := 1 To Size Do
    Begin
      C := S[I];
      Mem.Write( C, 1 );
    End;
End;

Procedure TPaper.ReadString( Mem : TMemoryStream; Var S : String; Size : Integer );
Var
  I : Integer;
  C : Char;
Begin
  S := '';
  For I := 1 To Size Do
    Begin
      Mem.Read( C, 1 );
      S := S + C;
    End;
End;

Procedure TPaper.SetUnits( Value : TUnits );
Begin
  If FUnits <> Value
    Then Begin
           FUnits := Value;
           Case FUnits Of
           unMillimeters      : CurrUnits := 1;
           unTenthMillimeters : CurrUnits := 0.1;
           unCentimeters      : CurrUnits := 10;
           unInches           : CurrUnits := MM_Per_Inch;
           unTenthInches      : CurrUnits := MM_Per_Inch/10;
           unHundredthInches  : CurrUnits := MM_Per_Inch/100;
           End;{Case }
           If Assigned( FChangeUnits )
             Then FChangeUnits( Self );
         End;
End;

Procedure TPaper.SetIndentions( Value : TIndentions );
Begin
  FIndentions := Value;
End;

Procedure TPaper.SetPrinterSetup( Value : TPrinterSetup );
Begin
  If FPrinterSetup <> Value
    Then FPrinterSetup := Value;
End;

Procedure TPaper.OnChangePrinterFormat( Sender : TObject );
Begin
  FWidth := Round( PrinterSetup.PaperWidth_Inch * Screen.PixelsPerInch );
  FHeight := Round( PrinterSetup.PaperHeight_Inch * Screen.PixelsPerInch );
  FPixelsPerMM.X := PrinterSetup.PixelsPerMM_byX;
  FPixelsPerMM.Y := PrinterSetup.PixelsPerMM_byY;
  If Assigned( FChangeSize )
    Then FChangeSize( Self );
End;

{ Public }

constructor TPaper.Create;
Begin
  Inherited Create;
  FWidth := 800;
  FHeight := 600;
  Try
    MemoryPaper := TMemoryStream.Create;
  Except
    WriteToLog( 'Unit <Paper.pas>: Error in creating object "MemoryPaper"' );
    Exit;
  End;
  Try
    SizesList := TList.Create;
  Except
    WriteToLog( 'Unit <Paper.pas>: Error in creating object "SizeList"' );
    Exit;
  End;
  Try
    Indentions := TIndentions.Create;
  Except
    WriteToLog( 'Unit <Paper.pas>: Error in creating object "Indentions"' );
    Exit;
  End;
  Try
    PrinterSetup := TPrinterSetup.Create;
  Except
    WriteToLog('Unit <Paper.pas>: Error in creating object "PrinterSetup"');
    Exit;
  End;
  CompressionQuality := 50;
  PrinterSetup.OnChangeFormat := OnChangePrinterFormat;
  PrinterSetup.PaperFormat := pfA4;
  ActivePage := 1;
  FUnits := unMillimeters;
  CurrUnits := 1;
  FPixelsPerMM.x := PrinterSetup.PixelsPerMM_byX;
  FPixelsPerMM.y := PrinterSetup.PixelsPerMM_byY;
End;

Destructor TPaper.Destroy;
Begin
  Clear;
  Try
    MemoryPaper.Clear;
  Except
    WriteToLog('Unit <Paper.pas>: Error in clearing object "MemoryPaper"');
  End;
  MemoryPaper.Free;
  SizesList.Free;
  Indentions.Free;
  PrinterSetup.Free;
  Inherited Destroy;
End;

Procedure TPaper.Clear;
Var
  I : Integer;
  Item : ^TStruct;
Begin
  With SizesList Do
    Begin
      For I := 0 To Count-1 Do
        Begin
          Item := Items[I];
          Dispose(Item);
        End;
      Clear;
    End;
  MemoryPaper.Clear;
End;

Procedure TPaper.DrawToCanvas( ACanvas : TCanvas; Zoom, Scale : Real; Page : SmallInt );
Var
  I    : Integer;
  Size : Integer;
  Item : ^TStruct;
  Mid  : Real;
  S    : String;
  Buff : Pointer;
  GT   : TGradType;
  TempStream         : TMemoryStream;
  Loc_Bitmap         : TBitmap;
  Loc_MetaFile       : TMetaFile;
  Loc_MetaFileCanvas : TMetaFileCanvas;
  Loc_Icon           : TIcon;
  Loc_JPEG           : TJpegImage;

{xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx}
    Procedure Draw_SetFont;
    Begin
      MemoryPaper.Read( varFont, Item^.HeaderSize );
      ACanvas.Font.Name := varFont.Name;
      Size := Round( varFont.Size * Scale/100 );
      ACanvas.Font.Size := Size;
      ACanvas.Font.Color := varFont.Color;
      ACanvas.Font.Style := [];
      If (varFont.Style And 1) = 1
        Then ACanvas.Font.Style := ACanvas.Font.Style + [fsBold];
      If (varFont.Style And 2) = 2
        Then ACanvas.Font.Style := ACanvas.Font.Style + [fsItalic];
      If (varFont.Style And 4) = 4
        Then ACanvas.Font.Style := ACanvas.Font.Style + [fsUnderline];
      If (varFont.Style And 8) = 8
        Then ACanvas.Font.Style := ACanvas.Font.Style + [fsStrikeOut];
    End;

    Procedure Draw_SetBrush;
    Begin
      MemoryPaper.Read( varBrush, Item^.HeaderSize );
      ACanvas.Brush.Color := varBrush.Color;
      Size := Item^.DataSize;
      If Size = 0
        Then Begin
               ACanvas.Brush.Bitmap := Nil;
               ACanvas.Brush.Color := varBrush.Color;
               Case TExBrushStyle(varBrush.Style) Of
               exSolid      : ACanvas.Brush.Style := bsSolid;
               exClear      : ACanvas.Brush.Style := bsClear;
               exHorizontal : ACanvas.Brush.Style := bsHorizontal;
               exVertical   : ACanvas.Brush.Style := bsVertical;
               exFDiagonal  : ACanvas.Brush.Style := bsFDiagonal;
               exBDiagonal  : ACanvas.Brush.Style := bsBDiagonal;
               exCross      : ACanvas.Brush.Style := bsCross;
               exDiagCross  : ACanvas.Brush.Style := bsDiagCross;
               Else ACanvas.Brush.Style := bsClear;
               End;{ case }
             End
        Else Begin
               Try
                 TempStream := TMemoryStream.Create;
                 Loc_Bitmap := TBitmap.Create;
                 GetMem( Buff, Size );
                 MemoryPaper.Read( Buff^, Size );
                 TempStream.Position := 0;
                 TempStream.Write( Buff^, Size );
                 TempStream.Position := 0;
                 Loc_Bitmap.LoadFromStream( TempStream );
                 ACanvas.Brush.Bitmap := TBitmap.Create;
                 ACanvas.Brush.Bitmap.Assign( Loc_Bitmap );
               Finally
                 FreeMem( Buff, Size );
                 Loc_Bitmap.Free;
                 TempStream.Free;
               End
             End;
    End;

    Procedure Draw_SetPen;
    Var
      PenWidth : Real;
    Begin
      MemoryPaper.Read( varPen, Item^.HeaderSize );
      Case varPen.Style Of
      exSolid : ACanvas.Pen.Style := psSolid;
      exClear : ACanvas.Pen.Style := psClear;
      End;{ case }
      ACanvas.Pen.Color := varPen.Color;
      Mid := (PixelsPerMM.X + PixelsPerMM.Y)/2;
      PenWidth := varPen.Width * MMPerDot;  // varPen.Width  -    
      PenWidth := Zoom * PenWidth * Mid * Scale/100;  //    
      ACanvas.Pen.Width := Round( PenWidth );
    End;

    Procedure Draw_Arc;
    Var
      X1, X2, X3, X4 : Integer;
      Y1, Y2, Y3, Y4 : Integer;
    Begin
      MemoryPaper.Read( varPie, Item^.HeaderSize );
      X1 := Round( Zoom * varPie.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varPie.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varPie.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varPie.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X3 := Round( Zoom * varPie.X3 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y3 := Round( Zoom * varPie.Y3 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X4 := Round( Zoom * varPie.X4 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y4 := Round( Zoom * varPie.Y4 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      ACanvas.Arc( X1, Y1, X2, Y2, X3, Y3, X4, Y4 );
    End;

    Procedure Draw_Chord;
    Var
      X1, X2, X3, X4 : Integer;
      Y1, Y2, Y3, Y4 : Integer;
      Loc_Rect       : TRect;
    Begin
      MemoryPaper.Read( varPie, Item^.HeaderSize );
      X1 := Round( Zoom * varPie.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varPie.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varPie.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varPie.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X3 := Round( Zoom * varPie.X3 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y3 := Round( Zoom * varPie.Y3 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X4 := Round( Zoom * varPie.X4 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y4 := Round( Zoom * varPie.Y4 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      If (TExBrushStyle(varBrush.Style) In GradientBrushesSet) And (Win32Platform = VER_PLATFORM_WIN32_NT)
        Then Begin
               Case TExBrushStyle(varBrush.Style) Of
               exLinear       : GT := gtLinear;
               exLinearDouble : GT := gtLinearDouble;
               exRadial       : GT := gtRadial;
               exRadialDouble : GT := gtRadialDouble;
               exSquare       : GT := gtSquare;
               exSquareDouble : GT := gtSquareDouble;
               exConic        : GT := gtConic;
               Else Exit;
               End;{ Case }
               Loc_MetaFile := TMetaFile.Create;
               Try
                 Loc_MetaFile.Width := Abs(X2-X1);
                 Loc_MetaFile.Height := Abs(Y2-Y1);
                 Try
                   Loc_MetaFileCanvas := TMetaFileCanvas.Create( Loc_MetaFile, 0 );
                 Except
                   WriteToLog( 'Unit <Paper.pas>: Error in creating object Loc_MetaFileCanvas in Draw_Chord procedure' );
                 End;
                 BeginPath( Loc_MetaFileCanvas.Handle );
                 Loc_MetaFileCanvas.Chord( 0, 0, X2-X1, Y2-Y1, X3-X1, Y3-Y1, X4-X1, Y4-Y1 );
                 EndPath( Loc_MetaFileCanvas.Handle );
                 SelectClipPath( Loc_MetaFileCanvas.Handle, RGN_AND );
                 Loc_Rect := Rect( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
                 GradientRect( Loc_MetaFileCanvas, Loc_Rect, Loc_Rect,
                               varBrush.Color, varBrush.Color2,
                               GT, varBrush.Params );
                 Loc_MetaFileCanvas.Free;
                 PlayEnhMetaFile( ACanvas.Handle, Loc_MetaFile.Handle, Rect(X1, Y1, X2, Y2) );
               Finally
                 ACanvas.Chord( X1, Y1, X2, Y2, X3, Y3, X4, Y4 );
                 Loc_MetaFile.Free;
               End;
             End
        Else Begin
               ACanvas.Chord( X1, Y1, X2, Y2, X3, Y3, X4, Y4 );
             End;
    End;

    Procedure Draw_Pie;
    Var
      X1, X2, X3, X4 : Integer;
      Y1, Y2, Y3, Y4 : Integer;
      Loc_Rect       : TRect;
    Begin
      MemoryPaper.Read( varPie, Item^.HeaderSize );
      X1 := Round( Zoom * varPie.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varPie.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varPie.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varPie.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X3 := Round( Zoom * varPie.X3 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y3 := Round( Zoom * varPie.Y3 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X4 := Round( Zoom * varPie.X4 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y4 := Round( Zoom * varPie.Y4 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      If (TExBrushStyle(varBrush.Style) In GradientBrushesSet) And (Win32Platform = VER_PLATFORM_WIN32_NT)
        Then Begin
               Case TExBrushStyle(varBrush.Style) Of
               exLinear       : GT := gtLinear;
               exLinearDouble : GT := gtLinearDouble;
               exRadial       : GT := gtRadial;
               exRadialDouble : GT := gtRadialDouble;
               exSquare       : GT := gtSquare;
               exSquareDouble : GT := gtSquareDouble;
               exConic        : GT := gtConic;
               Else Exit;
               End;{ Case }
               Loc_MetaFile := TMetaFile.Create;
               Try
                 Loc_MetaFile.Width := Abs(X2-X1);
                 Loc_MetaFile.Height := Abs(Y2-Y1);
                 Try
                   Loc_MetaFileCanvas := TMetaFileCanvas.Create( Loc_MetaFile, 0 );
                 Except
                   WriteToLog( 'Unit <Paper.pas>: Error in creating object Loc_MetaFileCanvas in Draw_Pie procedure' );
                 End;
                 BeginPath( Loc_MetaFileCanvas.Handle );
                 Loc_MetaFileCanvas.Pie( 0, 0, X2-X1, Y2-Y1, X3-X1, Y3-Y1, X4-X1, Y4-Y1 );
                 EndPath( Loc_MetaFileCanvas.Handle );
                 SelectClipPath( Loc_MetaFileCanvas.Handle, RGN_AND );
                 Loc_Rect := Rect( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
                 GradientRect( Loc_MetaFileCanvas, Loc_Rect, Loc_Rect,
                               varBrush.Color, varBrush.Color2,
                               GT, varBrush.Params );
                 Loc_MetaFileCanvas.Free;
                 PlayEnhMetaFile( ACanvas.Handle, Loc_MetaFile.Handle, Rect(X1, Y1, X2, Y2) );
               Finally
                 ACanvas.Pie( X1, Y1, X2, Y2, X3, Y3, X4, Y4 );
                 Loc_MetaFile.Free;
               End;
             End
        Else Begin
               ACanvas.Pie( X1, Y1, X2, Y2, X3, Y3, X4, Y4 );
             End;
    End;

    Procedure Draw_Draw;
    Var
      Loc_Rect : TRect;
      Ratio    : Real;
    Begin
      MemoryPaper.Read( varDraw, Item^.HeaderSize );
      Loc_Rect.Left := Round( Zoom * varDraw.X * PixelsPerMM.X * CurrUnits * Scale/100 );
      Loc_Rect.Top := Round( Zoom * varDraw.Y * PixelsPerMM.Y * CurrUnits * Scale/100 );
      Size := Item^.DataSize;
      TempStream := TMemoryStream.Create;
      GetMem( Buff, Size );
      Try
        TempStream.Position := 0;
        MemoryPaper.Read( Buff^, Size );
        TempStream.Position := 0;
        TempStream.Write( Buff^, Size );
        TempStream.Position := 0;
        Ratio := Zoom * (Scale/100);
        Case varDraw.GrType Of
        1 : Begin
              Loc_Bitmap := TBitmap.Create;
              Try
                Loc_Bitmap.LoadFromStream( TempStream );
                Loc_Rect.Right := Loc_Rect.Left + Round( Ratio * Loc_Bitmap.Width );
                Loc_Rect.Bottom := Loc_Rect.Top + Round( Ratio * Loc_Bitmap.Height );
                Loc_Bitmap.TransparentColor := varDraw.Color;
                Loc_Bitmap.Transparent := varDraw.Transp;
                ACanvas.StretchDraw( Loc_Rect, Loc_Bitmap );
              Finally
                Loc_Bitmap.Free;
              End;
            End;
        2 : Begin
              Loc_Icon := TIcon.Create;
              Try
                Loc_Icon.LoadFromStream( TempStream );
                Loc_Bitmap := TBitmap.Create;
                Try
                  Loc_Bitmap.Width := Loc_Icon.Width;
                  Loc_Bitmap.Height := Loc_Icon.Height;
                  Loc_Bitmap.Canvas.Draw( 0, 0, Loc_Icon );
                  Loc_Rect.Right := Loc_Rect.Left + Round( Ratio * Loc_Icon.Width);
                  Loc_Rect.Bottom := Loc_Rect.Top + Round( Ratio * Loc_Icon.Height);
                  ACanvas.StretchDraw( Loc_Rect, Loc_Bitmap );
                Finally
                  Loc_Bitmap.Free;
                End;  
              Finally
                Loc_Icon.Free;
              End;
            End;
        3 : Begin
              Loc_MetaFile := TMetaFile.Create;
              Try
                Loc_MetaFile.LoadFromStream( TempStream );
                Loc_Rect.Right := Loc_Rect.Left + Round( Ratio * Loc_MetaFile.Width);
                Loc_Rect.Bottom := Loc_Rect.Top + Round( Ratio * Loc_MetaFile.Height);
                ACanvas.StretchDraw( Loc_Rect, Loc_MetaFile );
              Finally
                Loc_MetaFile.Free;
              End;
            End;
        4 : Begin
              Loc_JPEG := TJPEGImage.Create;
              Try
                Loc_JPEG.LoadFromStream( TempStream );
                Loc_Rect.Right := Loc_Rect.Left + Round( Ratio * Loc_JPEG.Width );
                Loc_Rect.Bottom := Loc_Rect.Top + Round( Ratio * Loc_JPEG.Height );
                ACanvas.StretchDraw( Loc_Rect, Loc_JPEG );
              Finally
                Loc_JPEG.Free;
              End;
            End;
        End;{ case }
      Finally
        FreeMem( Buff, Size );
        TempStream.Free;
      End;
    End;

    Procedure Draw_StretchDraw;
    Var
      Loc_Rect : TRect;
    Begin
      MemoryPaper.Read( varStretchDraw, Item^.HeaderSize );
      Loc_Rect.Left := Round( Zoom * varStretchDraw.Rect.Left * PixelsPerMM.X * CurrUnits * Scale/100 );
      Loc_Rect.Top := Round( Zoom * varStretchDraw.Rect.Top * PixelsPerMM.Y * CurrUnits * Scale/100 );
      Loc_Rect.Right := Round( Zoom * varStretchDraw.Rect.Right * PixelsPerMM.X * CurrUnits * Scale/100 );
      Loc_Rect.Bottom := Round( Zoom * varStretchDraw.Rect.Bottom * PixelsPerMM.Y * CurrUnits * Scale/100 );
      TempStream := TMemoryStream.Create;
      Size := Item^.DataSize;
      GetMem( Buff, Size );
      Try
        MemoryPaper.Read( Buff^, Size );
        TempStream.Position := 0;
        TempStream.Write( Buff^, Size );
        TempStream.Position := 0;
        Case varStretchDraw.GrType Of
        1 : Begin
              Loc_Bitmap := TBitmap.Create;
              Try
                Loc_Bitmap.LoadFromStream( TempStream );
                Loc_Bitmap.TransparentColor := varDraw.Color;
                Loc_Bitmap.Transparent := varDraw.Transp;
                ACanvas.StretchDraw( Loc_Rect, Loc_Bitmap );
              Finally
                Loc_Bitmap.Free;
              End;
            End;
        2 : Begin
              Loc_Icon := TIcon.Create;
              Try
                Loc_Icon.LoadFromStream( TempStream );
                Loc_Bitmap := TBitmap.Create;
                Try
                  Loc_Bitmap.Width := Loc_Icon.Width;
                  Loc_Bitmap.Height := Loc_Icon.Height;
                  Loc_Bitmap.Canvas.Draw( 0, 0, Loc_Icon );
                  ACanvas.StretchDraw( Loc_Rect, Loc_Bitmap );
                Finally
                  Loc_Bitmap.Free;
                End;
              Finally
                Loc_Icon.Free;
              End;
            End;
        3 : Begin
              Loc_MetaFile := TMetaFile.Create;
              Try
                Loc_MetaFile.LoadFromStream( TempStream );
                ACanvas.StretchDraw( Loc_Rect, Loc_MetaFile );
              Finally
                Loc_MetaFile.Free;
              End;
            End;
        4 : Begin
              Loc_JPEG := TJPEGImage.Create;
              Try
                Loc_JPEG.LoadFromStream( TempStream );
                ACanvas.StretchDraw( Loc_Rect, Loc_JPEG );
              Finally
                Loc_JPEG.Free;
              End;
            End;
      End;{ case }
      Finally
        FreeMem( Buff, Size );
        TempStream.Free;
      End;
    End;

    Procedure Draw_Polygon;
    Var
      MinX, MinY : Integer;
      MaxX, MaxY : Integer;
      X1, X2, J  : Integer;
      Y1, Y2     : Integer;
      Loc_Points : Array Of TPoint;
      Loc_Rect   : TRect;
      Loc_RGN    : HRGN;
    Begin
      MemoryPaper.Read( varPoly.PointN, Item^.HeaderSize );
      SetLength( varPoly.Points, 0 );
      SetLength( varPoly.Points, varPoly.PointN );
      SetLength( Loc_Points, varPoly.PointN );
      For J := 0 To varPoly.PointN-1 Do
        Begin
          MemoryPaper.Read( varPoly.Points[J], SizeOf(TPoint) );
          Loc_Points[J].X := Round( Zoom * varPoly.Points[J].X * PixelsPerMM.X * CurrUnits * Scale/100 );
          Loc_Points[J].Y := Round( Zoom * varPoly.Points[J].Y * PixelsPerMM.Y * CurrUnits * Scale/100 );
        End;
      X1 := Loc_Points[0].X;
      Y1 := Loc_Points[0].Y;
      For J := 1 To varPoly.PointN-1 Do
        Begin
          If Loc_Points[J].X > X1
            Then X1 := Loc_Points[J].X;
          If Loc_Points[J].Y > Y1
            Then Y1 := Loc_Points[J].Y;
        End;
      MaxX := X1;
      MaxY := Y1;
      X2 := Loc_Points[0].X;
      Y2 := Loc_Points[0].Y;
      For J := 1 To varPoly.PointN-1 Do
        Begin
          If Loc_Points[J].X < X2
            Then X2 := Loc_Points[J].X;
          If Loc_Points[J].Y < Y2
            Then Y2 := Loc_Points[J].Y;
        End;
      MinX := X2;
      MinY := Y2;
      For J := 1 To varPoly.PointN Do
        Begin
          Rgn_Points[J].X := Loc_Points[J-1].X - MinX;
          Rgn_Points[J].Y := Loc_Points[J-1].Y - MinY;
        End;
      If TExBrushStyle(varBrush.Style) In GradientBrushesSet
        Then Begin
               Case TExBrushStyle(varBrush.Style) Of
               exLinear       : GT := gtLinear;
               exLinearDouble : GT := gtLinearDouble;
               exRadial       : GT := gtRadial;
               exRadialDouble : GT := gtRadialDouble;
               exSquare       : GT := gtSquare;
               exSquareDouble : GT := gtSquareDouble;
               exConic        : GT := gtConic;
               Else Exit;
               End;{ Case }
               Loc_Rect := Rect( 0, 0, Abs( MaxX - MinX ), Abs( MaxY - MinY ) );
               Loc_RGN := CreatePolygonRGN( Rgn_Points, varPoly.PointN, Alternate );
               Loc_MetaFile := TMetaFile.Create;
               Try
                 Loc_MetaFile.Width := Abs( MaxX - MinX );
                 Loc_MetaFile.Height := Abs( MaxY - MinY );
                 Loc_MetaFileCanvas := TMetaFileCanvas.Create( Loc_MetaFile, 0 );
                 Try
                   SelectClipRGN( Loc_MetaFileCanvas.Handle, Loc_RGN );
                   GradientRect( Loc_MetaFileCanvas, Loc_Rect, Loc_Rect,
                                 varBrush.Color, varBrush.Color2,
                                 GT, varBrush.Params );
                 Finally
                   Loc_MetaFileCanvas.Free;
                 End;
                 PlayEnhMetaFile( ACanvas.Handle, Loc_MetaFile.Handle, Rect(MinX, MinY, MaxX, MaxY) );
               Finally
                 ACanvas.Polygon( Loc_Points );
                 Loc_MetaFile.Free;
                 DeleteObject( Loc_RGN );
               End;
             End
        Else Begin
               ACanvas.Polygon( Loc_Points );
             End;
    End;

    Procedure Draw_PolyLine;
    Var
      Loc_Points : Array Of TPoint;
      J          : Integer;
    Begin
      MemoryPaper.Read( varPoly.PointN, Item^.HeaderSize );
      SetLength( varPoly.Points, 0 );
      SetLength( varPoly.Points, varPoly.PointN );
      SetLength( Loc_Points, varPoly.PointN );
      For J := 0 To varPoly.PointN-1 Do
        Begin
          MemoryPaper.Read( varPoly.Points[J], SizeOf(TPoint) );
          Loc_Points[J].X := Round( Zoom * varPoly.Points[J].X * PixelsPerMM.X * CurrUnits * Scale/100 );
          Loc_Points[J].Y := Round( Zoom * varPoly.Points[J].Y * PixelsPerMM.Y * CurrUnits * Scale/100 );
        End;
      ACanvas.PolyLine( Loc_Points );
    End;

    Procedure Draw_PolyBezier;
    Var
      Loc_Points : Array Of TPoint;
      J          : Integer;
    Begin
      MemoryPaper.Read( varPoly.PointN, Item^.HeaderSize );
      SetLength( varPoly.Points, 0 );
      SetLength( varPoly.Points, varPoly.PointN );
      SetLength( Loc_Points, varPoly.PointN );
      For J := 0 To varPoly.PointN-1 Do
        Begin
          MemoryPaper.Read( varPoly.Points[J], SizeOf(TPoint) );
          Loc_Points[J].X := Round( Zoom * varPoly.Points[J].X * PixelsPerMM.X * CurrUnits * Scale/100 );
          Loc_Points[J].Y := Round( Zoom * varPoly.Points[J].Y * PixelsPerMM.Y * CurrUnits * Scale/100 );
        End;
      ACanvas.Polybezier( Loc_Points );
    End;

    Procedure Draw_Ellipse;
    Var
      X1, X2   : Integer;
      Y1, Y2   : Integer;
      Loc_Rect : TRect;
      Loc_RGN  : HRGN;
    Begin
      MemoryPaper.Read( varRectangle, Item^.HeaderSize );
      X1 := Round( Zoom * varRectangle.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varRectangle.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varRectangle.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varRectangle.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      If TExBrushStyle(varBrush.Style) In GradientBrushesSet
        Then Begin
               Case TExBrushStyle(varBrush.Style) Of
               exLinear : GT := gtLinear;
               exLinearDouble : GT := gtLinearDouble;
               exRadial : GT := gtRadial;
               exRadialDouble : GT := gtRadialDouble;
               exSquare : GT := gtSquare;
               exSquareDouble : GT := gtSquareDouble;
               End; { Case }
               Loc_Rect := Rect( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
               Loc_RGN := CreateEllipticRGN( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
               Loc_MetaFile := TMetaFile.Create;
               Try
                 Loc_MetaFile.Width := Abs( X2 - X1 );
                 Loc_MetaFile.Height := Abs( Y2 - Y1 );
                 Loc_MetaFileCanvas := TMetaFileCanvas.Create( Loc_MetaFile, 0 );
                 Try
                   SelectClipRGN( Loc_MetaFileCanvas.Handle, Loc_RGN );
                   GradientRect( Loc_MetaFileCanvas, Loc_Rect, Loc_Rect,
                                 varBrush.Color, varBrush.Color2,
                                 GT, varBrush.Params );
                 Finally
                   Loc_MetaFileCanvas.Free;
                 End;
                 PlayEnhMetaFile( ACanvas.Handle, Loc_MetaFile.Handle, Rect(X1,Y1,X2,Y2) );
               Finally
                 ACanvas.Ellipse( X1, Y1, X2, Y2 );
                 Loc_MetaFile.Free;
                 DeleteObject( Loc_RGN );
               End;
             End
        Else Begin
               ACanvas.Ellipse( X1, Y1, X2, Y2 );
             End;
    End;

    Procedure Draw_Rectangle;
    Var
      X1, X2   : Integer;
      Y1, Y2   : Integer;
      Loc_Rect : TRect;
      Loc_RGN  : HRGN;
    Begin
      MemoryPaper.Read( varRectangle, Item^.HeaderSize );
      X1 := Round( Zoom * varRectangle.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varRectangle.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varRectangle.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varRectangle.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      If TExBrushStyle(varBrush.Style) In GradientBrushesSet
        Then Begin
               Case TExBrushStyle(varBrush.Style) Of
               exLinear : GT := gtLinear;
               exLinearDouble : GT := gtLinearDouble;
               exRadial : GT := gtRadial;
               exRadialDouble : GT := gtRadialDouble;
               exSquare : GT := gtSquare;
               exSquareDouble : GT := gtSquareDouble;
               End; { Case }
               Loc_Rect := Rect( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
               Loc_RGN := CreateRectRGN( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
               Loc_MetaFile := TMetaFile.Create;
               Try
                 Loc_MetaFile.Width := Abs( X2 - X1 );
                 Loc_MetaFile.Height := Abs( Y2 - Y1 );
                 Loc_MetaFileCanvas := TMetaFileCanvas.Create( Loc_MetaFile, 0 );
                 Try
                   SelectClipRGN( Loc_MetaFileCanvas.Handle, Loc_RGN );
                   GradientRect( Loc_MetaFileCanvas, Loc_Rect, Loc_Rect,
                                 varBrush.Color, varBrush.Color2,
                                 GT, varBrush.Params );
                 Finally
                   Loc_MetaFileCanvas.Free;
                 End;
                 PlayEnhMetaFile( ACanvas.Handle, Loc_MetaFile.Handle, Rect( X1, Y1, X2, Y2 ) );
               Finally
                 ACanvas.Rectangle( X1, Y1, X2, Y2 );
                 Loc_MetaFile.Free;
               End;
             End
        Else Begin
               ACanvas.Rectangle( X1, Y1, X2, Y2 );
             End;
    End;

    Procedure Draw_RoundRect;
    Var
      X1, Y1   : Integer;
      X2, Y2   : Integer;
      X3, Y3   : Integer;
      Loc_Rect : TRect;
      Loc_RGN  : HRGN;
    Begin
      MemoryPaper.Read( varRoundRect, Item^.HeaderSize );
      X1 := Round( Zoom * varRoundRect.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varRoundRect.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varRoundRect.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varRoundRect.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X3 := Round( Zoom * varRoundRect.X3 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y3 := Round( Zoom * varRoundRect.Y3 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      If TExBrushStyle(varBrush.Style) In GradientBrushesSet
        Then Begin
               Case TExBrushStyle(varBrush.Style) Of
               exLinear : GT := gtLinear;
               exLinearDouble : GT := gtLinearDouble;
               exRadial : GT := gtRadial;
               exRadialDouble : GT := gtRadialDouble;
               exSquare : GT := gtSquare;
               exSquareDouble : GT := gtSquareDouble;
               End; { Case }
               Loc_Rect := Rect( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
               Loc_RGN := CreateRoundRectRGN( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ), X3, Y3 );
               Loc_MetaFile := TMetaFile.Create;
               Try
                 Loc_MetaFile.Width := Abs( X2 - X1 );
                 Loc_MetaFile.Height := Abs( Y2 - Y1 );
                 Loc_MetaFileCanvas := TMetaFileCanvas.Create( Loc_MetaFile, 0 );
                 Try
                   SelectClipRGN( Loc_MetaFileCanvas.Handle, Loc_RGN );
                   GradientRect( Loc_MetaFileCanvas, Loc_Rect, Loc_Rect,
                                 varBrush.Color, varBrush.Color2,
                                 GT, varBrush.Params );
                 Finally
                   Loc_MetaFileCanvas.Free;
                 End;
                 PlayEnhMetaFile( ACanvas.Handle, Loc_MetaFile.Handle, Rect( X1, Y1, X2, Y2 ) );
               Finally
                 ACanvas.RoundRect( X1, Y1, X2, Y2, X3, Y3 );
                 Loc_MetaFile.Free;
                 DeleteObject( Loc_RGN );
               End;
             End
        Else Begin
               ACanvas.RoundRect( X1, Y1, X2, Y2, X3, Y3 );
             End;
    End;

    Procedure Draw_FrameRect;
    Var
      X1, X2 : Integer;
      Y1, Y2 : Integer;
    Begin
      MemoryPaper.Read( varRectangle, Item^.HeaderSize );
      X1 := Round( Zoom * varRectangle.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varRectangle.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varRectangle.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varRectangle.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      ACanvas.FrameRect( Rect(X1, Y1, X2, Y2) );
    End;

    Procedure Draw_FillRect;
    Var
      X1, X2   : Integer;
      Y1, Y2   : Integer;
      Loc_Rect : TRect;
      Loc_RGN  : HRGN;
    Begin
      MemoryPaper.Read( varRectangle, Item^.HeaderSize );
      X1 := Round( Zoom * varRectangle.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varRectangle.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varRectangle.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varRectangle.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      If TExBrushStyle(varBrush.Style) In GradientBrushesSet
        Then Begin
               Case TExBrushStyle(varBrush.Style) Of
               exLinear : GT := gtLinear;
               exLinearDouble : GT := gtLinearDouble;
               exRadial : GT := gtRadial;
               exRadialDouble : GT := gtRadialDouble;
               exSquare : GT := gtSquare;
               exSquareDouble : GT := gtSquareDouble;
               End; { Case }
               Loc_Rect := Rect( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
               Loc_RGN := CreateRectRGN( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
               Loc_MetaFile := TMetaFile.Create;
               Try
                 Loc_MetaFile.Width := Abs( X2 - X1 );
                 Loc_MetaFile.Height := Abs( Y2 - Y1 );
                 Loc_MetaFileCanvas := TMetaFileCanvas.Create( Loc_MetaFile, 0 );
                 Try
                   SelectClipRGN( Loc_MetaFileCanvas.Handle, Loc_RGN );
                   GradientRect( Loc_MetaFileCanvas, Loc_Rect, Loc_Rect,
                                 varBrush.Color, varBrush.Color2,
                                 GT, varBrush.Params );
                 Finally
                   Loc_MetaFileCanvas.Free;
                 End;
                 PlayEnhMetaFile( ACanvas.Handle, Loc_MetaFile.Handle, Rect( X1, Y1, X2, Y2 ) );
               Finally
                 Loc_MetaFile.Free;
               End;
             End
        Else Begin
               ACanvas.FillRect( Rect(X1, Y1, X2, Y2) );
             End;
    End;

    Procedure Draw_DrawFocusRect;
    Var
      X1, X2 : Integer;
      Y1, Y2 : Integer;
    Begin
      MemoryPaper.Read( varRectangle, Item^.HeaderSize );
      X1 := Round( Zoom * varRectangle.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varRectangle.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varRectangle.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varRectangle.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      ACanvas.DrawFocusRect( Rect(X1, Y1, X2, Y2) );
    End;

    Procedure Draw_Line;
    Var
      X1, X2 : Integer;
      Y1, Y2 : Integer;
    Begin
      MemoryPaper.Read( varRectangle, Item^.HeaderSize );
      X1 := Round( Zoom * varRectangle.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varRectangle.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varRectangle.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varRectangle.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      ACanvas.MoveTo( X1, Y1 );
      ACanvas.LineTo( X2, Y2 );
    End;

    Procedure Draw_TextRect;
    Var
      X1, Y1   : Integer;
      X2, Y2   : Integer;
      X3, Y3   : Integer;
      Loc_Rect : TRect;
    Begin
      MemoryPaper.Read( varTextRect, Item^.HeaderSize );
      ReadString( MemoryPaper, S, Item^.DataSize );
      X1 := Round( Zoom * varTextRect.Rect.Left * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varTextRect.Rect.Top * PixelsPerMM.X * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varTextRect.Rect.Right * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varTextRect.Rect.Bottom * PixelsPerMM.X * CurrUnits * Scale/100 );
      X3 := Round( Zoom * varTextRect.X * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y3 := Round( Zoom * varTextRect.Y * PixelsPerMM.Y * CurrUnits * Scale/100 );
      If TExBrushStyle(varBrush.Style) In GradientBrushesSet
        Then Begin
               Case TExBrushStyle(varBrush.Style) Of
               exLinear : GT := gtLinear;
               exLinearDouble : GT := gtLinearDouble;
               exRadial : GT := gtRadial;
               exRadialDouble : GT := gtRadialDouble;
               exSquare : GT := gtSquare;
               exSquareDouble : GT := gtSquareDouble;
               End; { Case }
               Loc_MetaFile := TMetaFile.Create;
               Try
                 Loc_MetaFile.Width := Abs( X2 - X1 );
                 Loc_MetaFile.Height := Abs( Y2 - Y1 );
                 Loc_MetaFileCanvas := TMetaFileCanvas.Create( Loc_MetaFile, 0 );
                 Try
                   Loc_MetaFileCanvas.Brush.Style := bsClear;
                   Loc_MetaFileCanvas.Font.Assign( ACanvas.Font );
                   BeginPath( Loc_MetaFileCanvas.Handle );
                   Loc_MetaFileCanvas.TextOut( X3 - X1, Y3 - Y1, S );
                   EndPath( Loc_MetaFileCanvas.Handle );
                   SelectClipPath( Loc_MetaFileCanvas.Handle, RGN_AND );
                   Loc_Rect := Rect( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
                   GradientRect( Loc_MetaFileCanvas, Loc_Rect, Loc_Rect,
                                 varBrush.Color, varBrush.Color2,
                                 GT, varBrush.Params );
                 Finally
                   Loc_MetaFileCanvas.Free;
                 End;
                   PlayEnhMetaFile( ACanvas.Handle, Loc_MetaFile.Handle, Rect( X1, Y1, X2, Y2 ) );
               Finally
                 Loc_MetaFile.Free;
               End;
             End
        Else Begin
               ACanvas.TextRect( Rect( X1, Y1, X2, Y2 ), X3, Y3, S );
             End;
    End;

    Procedure Draw_TextOut;
    Var
      X1, Y1   : Integer;
      Loc_Rect : TRect;
    Begin
      MemoryPaper.Read( varTextOut, Item^.HeaderSize );
      ReadString( MemoryPaper, S, Item^.DataSize );
      X1 := Round( Zoom * varTextOut.X * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varTextOut.Y * PixelsPerMM.X * CurrUnits * Scale/100 );
      If TExBrushStyle(varBrush.Style) In GradientBrushesSet
        Then Begin
               Case TExBrushStyle(varBrush.Style) Of
               exLinear : GT := gtLinear;
               exLinearDouble : GT := gtLinearDouble;
               exRadial : GT := gtRadial;
               exRadialDouble : GT := gtRadialDouble;
               exSquare : GT := gtSquare;
               exSquareDouble : GT := gtSquareDouble;
               End; { Case }
               Loc_Rect := Rect( X1, Y1, X1 + ACanvas.TextWidth( S ), Y1 + ACanvas.TextHeight( S ) );
               Loc_MetaFile := TMetaFile.Create;
               Try
                 Loc_MetaFile.Width := Abs( Loc_Rect.Right - Loc_Rect.Left );
                 Loc_MetaFile.Height := Abs( Loc_Rect.Bottom - Loc_Rect.Top );
                 Loc_MetaFileCanvas := TMetaFileCanvas.Create( Loc_MetaFile, 0 );
                 Try
                   Loc_MetaFileCanvas.Brush.Style := bsClear;
                   Loc_MetaFileCanvas.Font.Assign( ACanvas.Font );
                   Loc_MetaFileCanvas.Font.Height := ACanvas.Font.Height;
                   BeginPath( Loc_MetaFileCanvas.Handle );
                   Loc_MetaFileCanvas.TextOut( X1 - Loc_Rect.Left, Y1 - Loc_Rect.Top, S );
                   EndPath( Loc_MetaFileCanvas.Handle );
                   SelectClipPath( Loc_MetaFileCanvas.Handle, RGN_AND );
                   GradientRect( Loc_MetaFileCanvas,
                                 Rect( 0, 0, Abs(Loc_Rect.Right - Loc_Rect.Left), Abs( Loc_Rect.Bottom - Loc_Rect.Top )),
                                 Rect( 0, 0, Abs(Loc_Rect.Right - Loc_Rect.Left), Abs( Loc_Rect.Bottom - Loc_Rect.Top )),
                                 varBrush.Color, varBrush.Color2,
                                 GT, varBrush.Params );
                 Finally
                   Loc_MetaFileCanvas.Free;
                 End;
                   ACanvas.Draw( Loc_Rect.Left, Loc_Rect.Top, Loc_MetaFile );
               Finally
                 Loc_MetaFile.Free;
               End;
             End
        Else Begin
               ACanvas.TextOut( X1, Y1, S );
             End;
    End;

    Procedure Draw_Region;
    Var
      X1, Y1    : Integer;
      X2, Y2    : Integer;
      W, H      : Integer;
      Loc_Rect  : TRect;
      Loc_RGN   : HRGN;
      TempBrush : TBrush;
      RGNDATA   : PRGNDATA;
      Rect_Box  : TRect;
    Begin
      MemoryPaper.Read( varRectangle, Item^.HeaderSize );
      X1 := Round( Zoom * varRectangle.X1 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y1 := Round( Zoom * varRectangle.Y1 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      X2 := Round( Zoom * varRectangle.X2 * PixelsPerMM.X * CurrUnits * Scale/100 );
      Y2 := Round( Zoom * varRectangle.Y2 * PixelsPerMM.Y * CurrUnits * Scale/100 );
      GetMem( RGNDATA, Item^.DataSize );
      Try
        MemoryPaper.Read( RGNDATA^, Item^.DataSize );
        Loc_RGN := ExtCreateRegion( Nil, Item^.DataSize, RGNDATA^ );
      Finally
        FreeMem( RGNDATA, Item^.DataSize );
      End;
      GetRgnBox( Loc_RGN, Rect_Box );
      OffsetRGN( Loc_RGN, -Rect_Box.Left, -Rect_Box.Top );
      Loc_MetaFile := TMetaFile.Create;
      Try
        W := Abs( Rect_Box.Right - Rect_Box.Left );
        H := Abs( Rect_Box.Bottom - Rect_Box.Top );
        Loc_MetaFile.Width := W;
        Loc_MetaFile.Height := H;
        Loc_Rect := Rect( 0, 0, Abs( X2 - X1 ), Abs( Y2 - Y1 ) );
        Loc_MetaFileCanvas := TMetaFileCanvas.Create( Loc_MetaFile, 0 );
        Try
          SelectClipRGN( Loc_MetaFileCanvas.Handle, Loc_RGN );
          If TExBrushStyle(varBrush.Style) In GradientBrushesSet
            Then Begin
                   Case TExBrushStyle(varBrush.Style) Of
                   exLinear : GT := gtLinear;
                   exLinearDouble : GT := gtLinearDouble;
                   exRadial : GT := gtRadial;
                   exRadialDouble : GT := gtRadialDouble;
                   exSquare : GT := gtSquare;
                   exSquareDouble : GT := gtSquareDouble;
                   End; { Case }
                   GradientRect( Loc_MetaFileCanvas, Rect(0,0,W,H), Rect(0,0,W,H),
                                 varBrush.Color, varBrush.Color2,
                                 GT, varBrush.Params );
                 End
            Else Begin
                   Loc_MetaFileCanvas.Brush.Assign( ACanvas.Brush );
                   Loc_MetaFileCanvas.FillRect( Rect(0,0,W,H) );
                 End;
        If ACanvas.Pen.Style = psSolid
          Then Begin
                 TempBrush := TBrush.Create;
                 Try
                   TempBrush.Style := bsSolid;
                   TempBrush.Color := ACanvas.Pen.Color;
                   FrameRGN( Loc_MetaFileCanvas.Handle, Loc_RGN, TempBrush.Handle, ACanvas.Pen.Width, ACanvas.Pen.Width );
                 Finally
                   TempBrush.Free;
                 End;
               End;
        Finally
          Loc_MetaFileCanvas.Free;
        End;
        ACanvas.StretchDraw( Rect(X1, Y1, X2, Y2), Loc_MetaFile );
      Finally
        Loc_MetaFile.Free;
        DeleteObject( Loc_RGN );
      End;
    End;

{xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx}

{ DrawToCanvas }
Begin
  ReCountDataPos;
  MemoryPaper.Position := 0;
  SetDefault( ACanvas, Zoom, Scale );
  For I := 0 To SizesList.Count-1 Do
    Begin
      Item := SizesList.Items[I];
      MemoryPaper.Position := Item^.DataPos;
      Case Item^.CommandType Of
   cmSetDefault : Begin
                    SetDefault( ACanvas, Zoom, Scale );
                  End;
      cmSetFont : Begin
                    Draw_SetFont;
                  End;
     cmSetBrush : Begin
                    Draw_SetBrush;
                  End;
       cmSetPen : Begin
                    Draw_SetPen;
                  End;
      End;{ Case }
      If (Page <> Item^.PageNumber) And (Item^.PageNumber <> AllPages)
        Then Continue;
      MemoryPaper.Position := Item^.DataPos;
      Case Item^.CommandType Of
          cmArc : Begin
                    Draw_Arc;
                  End;
        cmChord : Begin
                    Draw_Chord;// gradient fill only under NT
                  End;
          cmPie : Begin
                    Draw_Pie;// gradient fill only under NT
                  End;
         cmDraw : Begin
                    Draw_Draw;
                  End;
  cmStretchDraw : Begin
                    Draw_StretchDraw;
                  End;
      cmPolygon : Begin
                    Draw_Polygon;
                  End;
     cmPolyline : Begin
                    Draw_PolyLine;
                  End;
   cmPolyBezier : Begin
                    Draw_PolyBezier;
                  End;
      cmEllipse : Begin
                    Draw_Ellipse;
                  End;
    cmRectangle : Begin
                    Draw_Rectangle;
                  End;
    cmRoundRect : Begin
                    Draw_RoundRect;
                  End;
    cmFrameRect : Begin
                    Draw_FrameRect;
                  End;
     cmFillRect : Begin
                    Draw_FillRect;
                  End;
cmDrawFocusRect : Begin
                    Draw_DrawFocusRect;
                  End;
         cmLine : Begin
                    Draw_Line;
                  End;
     cmTextRect : Begin
                    Draw_TextRect;
                  End;
     cmTextOut  : Begin
                    Draw_TextOut;
                  End;
       cmRegion : Begin
                    Draw_Region;
                  End;
      End;{ case }
      If Assigned( FOnDrawItem )
        Then FOnDrawItem( Self, Item^.CommandType, I );
    End;
End;


procedure TPaper.Arc( X1, Y1, X2, Y2, X3, Y3, X4, Y4 : Integer );
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varPie.X1 := X1;
  varPie.X2 := X2;
  varPie.X3 := X3;
  varPie.X4 := X4;
  varPie.Y1 := Y1;
  varPie.Y2 := Y2;
  varPie.Y3 := Y3;
  varPie.Y4 := Y4;
  Size := SizeOf(varPie);
  New( Item );
  Item^.CommandType := cmArc;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varPie, Size );
End;

procedure TPaper.BrushCopy( const Dest : TRect; Bitmap : TBitmap;
                            const Source : TRect; Color : TColor );
Var
  B    : TBitmap;
Begin
  B := TBitmap.Create;
  Try
    B.PixelFormat := pf24bit;
    B.Width := Abs( Source.Right - Source.Left );
    B.Height := Abs( Source.Right - Source.Left );
    B.Canvas.BrushCopy( Rect(0,0,B.Width,B.Height), Bitmap, Source, Color );
    StretchDraw( Dest, B );
  Finally
    B.Free;
  End;
End;

procedure TPaper.Chord( X1, Y1, X2, Y2, X3, Y3, X4, Y4 : Integer );
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varPie.X1 := X1;
  varPie.X2 := X2;
  varPie.X3 := X3;
  varPie.X4 := X4;
  varPie.Y1 := Y1;
  varPie.Y2 := Y2;
  varPie.Y3 := Y3;
  varPie.Y4 := Y4;
  Size := SizeOf(varPie);
  New( Item );
  Item^.CommandType := cmChord;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varPie, Size );
End;

procedure TPaper.Draw( X, Y : Integer; Graphic : TGraphic );
Var
  Item : ^TStruct;
  Temp : TMemoryStream;
  Buff : Pointer;
  Size : Integer;
Begin
  varDraw.X := X;
  varDraw.Y := Y;
  varDraw.Width := Graphic.Width;
  varDraw.Height := Graphic.Height;
  Size := SizeOf( varDraw );
  If Graphic Is TBitmap
    Then varDraw.GrType := 1
    Else If Graphic Is TIcon
           Then varDraw.GrType := 2
           Else If Graphic Is TMetaFile
                  Then varDraw.GrType := 3
                  Else If Graphic Is TJpegImage
                         Then varDraw.GrType := 4
                         Else varDraw.GrType := 0;
  If Graphic Is TBitmap
    Then Begin
           varDraw.Transp := (Graphic As TBitmap).Transparent;
           varDraw.Color := (Graphic As TBitmap).TransparentColor;
         End
    Else varDraw.Transp := False;
  New( Item );
  Item^.CommandType := cmDraw;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  MemoryPaper.Write( varDraw, Size );

  Temp := TMemoryStream.Create;
  Graphic.SaveToStream( Temp );
  Size := Temp.Size;
  GetMem( Buff, Size );
  Temp.Position := 0;
  Temp.Read( Buff^, Size );
  MemoryPaper.Write( Buff^, Size );
  FreeMem( Buff, Size );
  Temp.Free;

  Item^.DataSize := Size;
  SizesList.Add( Item );
End;

procedure TPaper.DrawFocusRect( const Rect: TRect );
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varRectangle.X1 := Rect.Left;
  varRectangle.Y1 := Rect.Top;
  varRectangle.X2 := Rect.Right;
  varRectangle.Y2 := Rect.Bottom;
  Size := SizeOf(varRectangle);
  New( Item );
  Item^.CommandType := cmDrawFocusRect;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.DataSize := 0;
  Item^.HeaderSize := Size;
  SizesList.Add( Item );
  MemoryPaper.Write( varRectangle, Size );
End;

procedure TPaper.Ellipse( X1, Y1, X2, Y2 : Integer );
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varRectangle.X1 := X1;
  varRectangle.Y1 := Y1;
  varRectangle.X2 := X2;
  varRectangle.Y2 := Y2;
  Size := SizeOf(varRectangle);
  New( Item );
  Item^.CommandType := cmEllipse;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varRectangle, Size );
End;

procedure TPaper.Ellipse(const Rect: TRect);
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varRectangle.X1 := Rect.Left;
  varRectangle.Y1 := Rect.Top;
  varRectangle.X2 := Rect.Right;
  varRectangle.Y2 := Rect.Bottom;
  Size := SizeOf(varRectangle);
  New( Item );
  Item^.CommandType := cmEllipse;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varRectangle, Size );
End;

procedure TPaper.FillRect(const Rect: TRect);
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varRectangle.X1 := Rect.Left;
  varRectangle.Y1 := Rect.Top;
  varRectangle.X2 := Rect.Right;
  varRectangle.Y2 := Rect.Bottom;
  Size := SizeOf(varRectangle);
  New( Item );
  Item^.CommandType := cmFillRect;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varRectangle, Size );
End;

procedure TPaper.FrameRect(const Rect: TRect);
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varRectangle.X1 := Rect.Left;
  varRectangle.Y1 := Rect.Top;
  varRectangle.X2 := Rect.Right;
  varRectangle.Y2 := Rect.Bottom;
  Size := SizeOf(varRectangle);
  New( Item );
  Item^.CommandType := cmFrameRect;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varRectangle, Size );
End;

procedure TPaper.Line( X1, Y1, X2, Y2 : Integer );
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varRectangle.X1 := X1;
  varRectangle.Y1 := Y1;
  varRectangle.X2 := X2;
  varRectangle.Y2 := Y2;
  Size := SizeOf(varRectangle);
  New( Item );
  Item^.CommandType := cmLine;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varRectangle, Size );
End;

procedure TPaper.Pie( X1, Y1, X2, Y2, X3, Y3, X4, Y4 : Integer );
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varPie.X1 := X1;
  varPie.X2 := X2;
  varPie.X3 := X3;
  varPie.X4 := X4;
  varPie.Y1 := Y1;
  varPie.Y2 := Y2;
  varPie.Y3 := Y3;
  varPie.Y4 := Y4;
  Size := SizeOf(varPie);
  New( Item );
  Item^.CommandType := cmPie;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varPie, Size );
End;

procedure TPaper.Polygon(const Points: array of TPoint);
Var
  Item : ^TStruct;
  Size : Integer;
  I, N : Integer;
Begin
  N := High(Points) + 1;
  varPoly.PointN := N;
  MemoryPaper.Write( varPoly.PointN, SizeOf(Integer) );
  SetLength( varPoly.Points, N );
  For I := 0 To N-1 Do
    varPoly.Points[I] := Points[I];
  Size := SizeOf(TPoint)*N;
  New( Item );
  Item^.CommandType := cmPolygon;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := 4;
  Item^.DataSize := Size;
  SizesList.Add( Item );
  For I := 0 To N-1 Do
    MemoryPaper.Write( varPoly.Points[I], SizeOf(TPoint) );
End;

procedure TPaper.Polyline(const Points: array of TPoint);
Var
  Item : ^TStruct;
  Size : Integer;
  I, N : Integer;
Begin
  N := High(Points) + 1;
  varPoly.PointN := N;
  MemoryPaper.Write( varPoly.PointN, SizeOf(Integer) );
  SetLength( varPoly.Points, N );
  For I := 0 To N-1 Do
    varPoly.Points[I] := Points[I];
  Size := SizeOf(TPoint)*N;
  New( Item );
  Item^.CommandType := cmPolyLine;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := SizeOf(Integer);
  Item^.DataSize := Size;
  SizesList.Add( Item );
  For I := 0 To N-1 Do
    MemoryPaper.Write( varPoly.Points[I], SizeOf(TPoint) );
End;

procedure TPaper.PolyBezier(const Points: array of TPoint);
Var
  Item : ^TStruct;
  Size : Integer;
  I, N : Integer;
Begin
  N := High(Points) + 1;
  varPoly.PointN := N;
  MemoryPaper.Write( varPoly.PointN, SizeOf(Integer) );
  SetLength( varPoly.Points, N );
  For I := 0 To N-1 Do
    varPoly.Points[I] := Points[I];
  Size := SizeOf(TPoint)*N;
  New( Item );
  Item^.CommandType := cmPolyBezier;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := SizeOf(Integer);
  Item^.DataSize := Size;
  SizesList.Add( Item );
  For I := 0 To N-1 Do
    MemoryPaper.Write( varPoly.Points[I], SizeOf(TPoint) );
End;

procedure TPaper.Rectangle( X1, Y1, X2, Y2 : Integer );
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varRectangle.X1 := X1;
  varRectangle.Y1 := Y1;
  varRectangle.X2 := X2;
  varRectangle.Y2 := Y2;
  Size := SizeOf( varRectangle );
  New( Item );
  Item^.CommandType := cmRectangle;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varRectangle, Size );
End;

procedure TPaper.Rectangle(const Rect: TRect);
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varRectangle.X1 := Rect.Left;
  varRectangle.Y1 := Rect.Top;
  varRectangle.X2 := Rect.Right;
  varRectangle.Y2 := Rect.Bottom;
  Size := SizeOf(varRectangle);
  New( Item );
  Item^.CommandType := cmRectangle;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varRectangle, Size );
End;

procedure TPaper.RoundRect( X1, Y1, X2, Y2, X3, Y3 : Integer );
Var
  Item : ^TStruct;
  Size : Integer;
Begin
  varRoundRect.X1 := X1;
  varRoundRect.Y1 := Y1;
  varRoundRect.X2 := X2;
  varRoundRect.Y2 := Y2;
  varRoundRect.X3 := X3;
  varRoundRect.Y3 := Y3;
  Size := SizeOf(varRoundRect);
  New( Item );
  Item^.CommandType := cmRoundRect;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varRoundRect, Size );
End;

procedure TPaper.StretchDraw(const Rect: TRect; Graphic: TGraphic);
Var
  Item : ^TStruct;
  Temp : TMemoryStream;
  Buff : Pointer;
  Size : Integer;
Begin
  varStretchDraw.Rect := Rect;
  Size := SizeOf( varStretchDraw );
  If Graphic Is TBitmap
    Then varStretchDraw.GrType := 1
    Else If Graphic Is TIcon
           Then varStretchDraw.GrType := 2
           Else If Graphic Is TMetaFile
                  Then varStretchDraw.GrType := 3
                  Else If Graphic Is TJPEGImage
                         Then varStretchDraw.GrType := 4
                         Else varStretchDraw.GrType := 0;
  If Graphic Is TBitmap
    Then Begin
           varDraw.Transp := (Graphic As TBitmap).Transparent;
           varDraw.Color := (Graphic As TBitmap).TransparentColor;
         End
    Else varDraw.Transp := False;
  New( Item );
  Item^.CommandType := cmStretchDraw;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  MemoryPaper.Write( varStretchDraw, Size );

  Temp := TMemoryStream.Create;
  Graphic.SaveToStream( Temp );
  Size := Temp.Size;
  GetMem( Buff, Size );
  Temp.Position := 0;
  Temp.Read( Buff^, Size );
  MemoryPaper.Write( Buff^, Size );
  FreeMem( Buff, Size );
  Temp.Free;

  Item^.DataSize := Size;
  SizesList.Add( Item );
End;

function TPaper.TextExtent( Const Text : String ) : TSize;
Var
  B : TBitmap;
  Z : TSize;
Begin
  B := TBitmap.Create;
  Try
    B.Width := 100;
    B.Height := 100;
    B.Canvas.Font.Name := varFont.Name;
    B.Canvas.Font.Size := varFont.Size;
    B.Canvas.Font.Style := [];
    If (varFont.Style And 1) = 1
      Then B.Canvas.Font.Style := B.Canvas.Font.Style + [fsBold];
    If (varFont.Style And 2) = 2
      Then B.Canvas.Font.Style := B.Canvas.Font.Style + [fsItalic];
    If (varFont.Style And 4) = 4
      Then B.Canvas.Font.Style := B.Canvas.Font.Style + [fsUnderline];
    If (varFont.Style And 8) = 8
      Then B.Canvas.Font.Style := B.Canvas.Font.Style + [fsStrikeOut];
    Z := B.Canvas.TextExtent( Text );
    Result.cx := Round((Z.CX * PrinterSetup.PaperWidth_MM / Width) / CurrUnits );
    Result.cy := Round((Z.CY * PrinterSetup.PaperHeight_MM / Height) / CurrUnits);
  Finally
    B.Free;
  End;
End;

function TPaper.TextWidth( const Text : string ) : Integer;
Begin
  Result := TextExtent(Text).CX;
End;

function TPaper.TextHeight( const Text : string ) : Integer;
Begin
  Result := TextExtent(Text).CY;
End;

procedure TPaper.TextOut( X, Y : Integer; const Text : string );
Var
  Size : Integer;
  Item : ^TStruct;
Begin
  varTextOut.X := X;
  varTextOut.Y := Y;
  Size := SizeOf( X ) + SizeOf( Y );
  MemoryPaper.Write( varTextOut, Size );
  New( Item );
  Item^.CommandType := cmTextOut;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Size := Length(Text);
  WriteString( MemoryPaper, Text, Size );
  Item^.DataSize := Size;
  SizesList.Add( Item );
End;

procedure TPaper.TextRect( Rect : TRect; X, Y : Integer; const Text: string) ;
Var
  Size : Integer;
  Item : ^TStruct;
Begin
  varTextRect.Rect := Rect;
  varTextRect.X := X; //mm
  varTextRect.Y := Y;
  Size := SizeOf( Rect )+ SizeOf( X ) + SizeOf( Y );
  MemoryPaper.Write( varTextRect, Size );
  New( Item );
  Item^.CommandType := cmTextRect;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Size := Length(Text);
  WriteString( MemoryPaper, Text, Size );
  Item^.DataSize := Size;
  SizesList.Add( Item );
End;

procedure TPaper.DrawRegion( RGN : HRGN );
Var
  Size : Integer;
  Item : ^TStruct;
  Buff : PRGNDATA;
  ARect : TRect;
Begin
  GetRgnBox( RGN, ARect );
  varRectangle.X1 := ARect.Left;
  varRectangle.Y1 := ARect.Top;
  varRectangle.X2 := ARect.Right;
  varRectangle.Y2 := ARect.Bottom;
  New( Item );
  Item^.CommandType := cmRegion;
  Item^.PageNumber := ActivePage;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := SizeOf(varRectangle);
  MemoryPaper.Write( varRectangle, SizeOf(varRectangle) );
  Size := GetRegionData( RGN, 0, Nil );
  Item^.DataSize := Size;
  GetMem( Buff, Size );
  Try
    GetRegionData( RGN, Size, Buff );
    MemoryPaper.Write( Buff^, Size );
  Finally
    FreeMem( Buff, Size );
  End;
  SizesList.Add( Item );
End;

Procedure TPaper.SetFont( Name : String; Size : Integer; Style : TFontStyles; Color : TColor );
Var
  Item : ^TStruct;
  Size1 : Integer;
Begin
  varFont.Name := Name;
  varFont.Size := Size;
  varFont.Color := Color;
  varFont.Style := 0;
  If fsBold In Style
    Then varFont.Style := varFont.Style Or 1;
  If fsItalic In Style
    Then varFont.Style := varFont.Style Or 2;
  If fsUnderline In Style
    Then varFont.Style := varFont.Style Or 4;
  If fsStrikeOut In Style
    Then varFont.Style := varFont.Style Or 8;
  Size1 := SizeOf( varFont );
  New( Item );
  Item^.CommandType := cmSetFont;
  Item^.PageNumber := AllPages;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size1;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varFont, Size1 );
End;

Procedure TPaper.SetBrush( Style: TExBrushStyle; Color, Color2 : TColor; Bitmap : TBitmap; Params : Byte );
Var
  Size : Integer;
  Item : ^TStruct;
  Temp : TMemoryStream;
  Buff : Pointer;
Begin
  varBrush.Style := Byte(Style);
  varBrush.Color := Color;
  varBrush.Color2 := Color2;
  varBrush.Params := Params;
  Size := SizeOf( varBrush );
  New( Item );
  Item^.CommandType := cmSetBrush;
  Item^.PageNumber := AllPages;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  MemoryPaper.Write( varBrush, Size );
  If Bitmap = Nil
    Then Begin
           Item^.DataSize := 0;
           SizesList.Add( Item );
         End
    Else Begin
           Temp := TMemoryStream.Create;
           Bitmap.SaveToStream( Temp );
           Size := Temp.Size;
           GetMem( Buff, Size );
           Temp.Position := 0;
           Temp.Read( Buff^, Size );
           MemoryPaper.Write( Buff^, Size );
           FreeMem( Buff, Size );
           Temp.Free;
           Item^.DataSize := Size;
           SizesList.Add( Item );
         End;
End;

Procedure TPaper.SetPen( Style : TExBrushStyle; Color : TColor; Width : Real );
Var
  Size : Integer;
  Item : ^TStruct;
Begin
  varPen.Color := Color;
  varPen.Width := Width;
  varPen.Style := Style;
  Size := SizeOf( varPen );
  New( Item );
  Item^.CommandType := cmSetPen;
  Item^.PageNumber := AllPages;
  Item^.DataPos := MemoryPaper.Position;
  Item^.HeaderSize := Size;
  Item^.DataSize := 0;
  SizesList.Add( Item );
  MemoryPaper.Write( varPen, Size );
End;

{*************}


Procedure TPaper.ReCountDataPos;
Var
  I, DPos : Integer;
  Item : ^TStruct;
Begin
  DPos := 0;
  For I := 0 To SizesList.Count-1 Do
    Begin
      Item := SizesList.Items[I];
      Item^.DataPos := DPos;
      SizesList.Items[I] := Item;
      DPos := DPos + Item^.HeaderSize + Item^.DataSize;
    End;
End;

Procedure TPaper.SetDefault( ACanvas : TCanvas; Zoom, Scale : Real  );
Var
  Mid : Real;
  WMM : Real;
Begin
  Mid := (PixelsPerMM.X + PixelsPerMM.Y)/2;
  WMM := 0.5 * MMPerDot;  // 0.5  -    
  WMM := Zoom * Mid * WMM;  //    

  ACanvas.Font.Name := 'Times New Roman';
  ACanvas.Font.Style := [];
  ACanvas.Font.Color := clBlack;
  ACanvas.Font.Size := Round(10*Scale/100);
  ACanvas.PenPos := Point(0,0);
  ACanvas.Pen.Style := psSolid;
  ACanvas.Pen.Mode := pmCopy;
  ACanvas.Pen.Color := clBlack;
  ACanvas.Pen.Width := Round( WMM );
  ACanvas.Brush.Bitmap := Nil;
  ACanvas.Brush.Color := clWhite;
  ACanvas.Brush.Style := bsClear;

  ActivePage := 1;
  varFont.Name := 'Times New Roman';
  varFont.Size := 10;
  varFont.Color := clBlack;
  varFont.Style := 0;
  varPen.Style := exClear;
  varPen.Color := clBlack;
  varPen.Width := 0.5;
  varBrush.Style := Byte(exClear);
  varBrush.Color := clWhite;
  varBrush.Color2 := clWhite;
  varBrush.Params := 0;
End;

Function TPaper.GetItemsCount : Integer;
Begin
  Result := SizesList.Count;
End;

Function TPaper.GetItemsCountOnPage( PageNumber : SmallInt ) : Integer;
Var
  N, I : Integer;
  Item : ^TStruct;
Begin
  N := 0;
  For I := 0 To SizesList.Count-1 Do
    Begin
      Item := SizesList.Items[I];
      If (Item^.PageNumber = PageNumber) Or (Item^.PageNumber = AllPages)
        Then Inc( N );
    End;
  Result := N;
End;

Procedure TPaper.SaveToFile( FileName : String; WriteMethod : TRWMethod );
Var
  FF     : TFileStream;
  Header : TFileHeader;
  OldH   : TFileHeader;
  I      : Integer;
  Item   : ^TStruct;
  Empty  : TStruct;
  IsAdd  : Boolean;
Begin
  Empty.CommandType := cmSetDefault;
  Empty.PageNumber := 1;
  Empty.DataPos := SizeOf(TStruct);
  Empty.HeaderSize := 0;
  Empty.DataSize := 0;
  If SizesList.Count > 0
    Then Begin
           Item := SizesList.Items[0];
           If Item^.CommandType <> cmSetDefault
             Then IsAdd := True
             Else IsAdd := False;
         End
    Else Begin
           IsAdd := True;
         End;
  If WriteMethod = PAGE_NEW
    Then Begin
           FF := TFileStream.Create( FileName, fmCreate );
           Try
             Header.PreviewMark := 'PRV';
             Header.Version := 300;
             If IsAdd
               Then Header.ItemsCount := SizesList.Count + 1
               Else Header.ItemsCount := SizesList.Count;
             Header.DataPos := SizeOf( TFileHeader );
             Header.PaperFormat := PrinterSetup.PaperFormat;
             Header.PaperWidth := PrinterSetup.PaperWidth_MM;
             Header.PaperHeight := PrinterSetup.PaperHeight_MM;
             Header.Units := Self.Units;
             Header.Orientation := PrinterSetup.Orientation;
             FF.Write( Header, SizeOf( TFileHeader ) );
             ReCountDataPos;
             If IsAdd
               Then Begin
                      Empty.DataPos := Header.DataPos;
                      FF.Write( Empty, SizeOf(TStruct) );
                    End;
             For I := 0 To SizesList.Count-1 Do
               Begin
                 Item := SizesList.Items[I];
                 MemoryPaper.Seek( Item^.DataPos, soFromBeginning );
                 Item^.DataPos := FF.Position + SizeOf(TStruct);
                 FF.Write( Item^, SizeOf(TStruct) );
                 If Item^.HeaderSize + Item^.DataSize > 0
                   Then FF.CopyFrom( MemoryPaper, Item^.HeaderSize + Item^.DataSize );
               End;
           Finally
             FF.Free;
           End;
         End
    Else Begin
           If FileExists( FileName )
             Then FF := TFileStream.Create( FileName, fmOpenReadWrite )
             Else FF := TFileStream.Create( FileName, fmCreate );
           Try
             FF.Seek( 0, soFromBeginning );
             FF.Read( OldH, SizeOf(TFileHeader) );
             If (OldH.PreviewMark <> 'PRV') Or (OldH.Version > 300)
               Then Begin
                      raise Exception.Create('Format error');
                    End;
             Header.PreviewMark := 'PRV';
             Header.Version := 300;
             If IsAdd
               Then Header.ItemsCount := OldH.ItemsCount + SizesList.Count + 1
               Else Header.ItemsCount := OldH.ItemsCount + SizesList.Count;
             Header.DataPos := SizeOf( TFileHeader );
             FF.Seek( 0, soFromBeginning );
             FF.Write( Header, SizeOf( TFileHeader ) );
             FF.Seek( 0, soFromEnd );
             ReCountDataPos;
             If IsAdd
               Then Begin
                      Empty.DataPos := Header.DataPos;
                      FF.Write( Empty, SizeOf(TStruct) );
                    End;
             For I := 0 To SizesList.Count-1 Do
               Begin
                 Item := SizesList.Items[I];
                 MemoryPaper.Seek( Item^.DataPos, soFromBeginning );
                 Item^.DataPos := FF.Position + SizeOf(TStruct);
                 FF.Write( Item^, SizeOf(TStruct) );
                 If Item^.HeaderSize + Item^.DataSize > 0
                   Then FF.CopyFrom( MemoryPaper, Item^.HeaderSize + Item^.DataSize );
               End;
           Finally
             FF.Free;
           End;
         End;
End;

Procedure TPaper.LoadFromFile( FileName : String; ReadMethod : TRWMethod );
Var
  FF     : TFileStream;
  Header : TFileHeader;
  I      : Integer;
  Item   : ^TStruct;
Begin
  FF := TFileStream.Create( FileName, fmOpenRead );
  Try
    FF.Seek( 0, soFromBeginning );
    FF.Read( Header, SizeOf(TFileHeader) );
    If (Header.PreviewMark <> 'PRV') Or (Header.Version > 300)
      Then Begin
             raise Exception.Create('Format error');
           End;
    FF.Seek( Header.DataPos, soFromBeginning );
    If ReadMethod = PAGE_NEW
      Then Begin
             Clear;
             If Header.PaperFormat = pfCustom
               Then Begin
                      PrinterSetup.PaperWidth_MM := Header.PaperWidth;
                      PrinterSetup.PaperHeight_MM := Header.PaperHeight;
                    End
               Else Begin
                      PrinterSetup.PaperFormat := Header.PaperFormat;
                    End;
             Self.Units := Header.Units;
             PrinterSetup.Orientation := Header.Orientation;
           End;
    SizesList.Capacity := SizesList.Count + Header.ItemsCount;
    MemoryPaper.Position := MemoryPaper.Size;
    For I := 0 To Header.ItemsCount-1 Do
      Begin
        New( Item );
        FF.Read( Item^, SizeOf(TStruct) );
        If Item^.HeaderSize + Item^.DataSize > 0
          Then MemoryPaper.CopyFrom( FF, Item^.HeaderSize + Item^.DataSize );
        SizesList.Add( Item );
      End;
    ReCountDataPos;
  Finally
    FF.Free;
  End;
End;

Procedure TPaper.SavePageToFile( FileName : String; Page : SmallInt; WriteMethod : TRWMethod );
Var
  FF     : TFileStream;
  Header : TFileHeader;
  OldH   : TFileHeader;
  I      : Integer;
  Item   : ^TStruct;
  Empty  : TStruct;
  IsAdd  : Boolean;
Begin
  Empty.CommandType := cmSetDefault;
  Empty.PageNumber := 1;
  Empty.DataPos := SizeOf(TStruct);
  Empty.HeaderSize := 0;
  Empty.DataSize := 0;
  If SizesList.Count > 0
    Then Begin
           Item := SizesList.Items[0];
           If Item^.CommandType <> cmSetDefault
             Then IsAdd := True
             Else IsAdd := False;
         End
    Else Begin
           IsAdd := True;
         End;
  If WriteMethod = PAGE_NEW
    Then Begin
           FF := TFileStream.Create( FileName, fmCreate );
           Try
             Header.PreviewMark := 'PRV';
             Header.Version := 300;
             If IsAdd
               Then Header.ItemsCount := SizesList.Count + 1
               Else Header.ItemsCount := SizesList.Count;
             Header.DataPos := SizeOf( TFileHeader );
             Header.PaperFormat := PrinterSetup.PaperFormat;
             Header.PaperWidth := PrinterSetup.PaperWidth_MM;
             Header.PaperHeight := PrinterSetup.PaperHeight_MM;
             Header.Units := Self.Units;
             Header.Orientation := PrinterSetup.Orientation;
             FF.Write( Header, SizeOf( TFileHeader ) );
             ReCountDataPos;
             If IsAdd
               Then Begin
                      Empty.DataPos := Header.DataPos;
                      FF.Write( Empty, SizeOf(TStruct) );
                    End;
             For I := 0 To SizesList.Count-1 Do
               Begin
                 Item := SizesList.Items[I];
                 If (Item^.PageNumber = Page) Or (Item^.PageNumber = AllPages)
                   Then Begin
                          MemoryPaper.Seek( Item^.DataPos, soFromBeginning );
                          Item^.DataPos := FF.Position + SizeOf(TStruct);
                          FF.Write( Item^, SizeOf(TStruct) );
                          If Item^.HeaderSize + Item^.DataSize > 0
                            Then FF.CopyFrom( MemoryPaper, Item^.HeaderSize + Item^.DataSize );
                        End;  
               End;
           Finally
             FF.Free;
           End;
         End
    Else Begin
           If FileExists( FileName )
             Then FF := TFileStream.Create( FileName, fmOpenReadWrite )
             Else FF := TFileStream.Create( FileName, fmCreate );
           Try
             FF.Seek( 0, soFromBeginning );
             FF.Read( OldH, SizeOf(TFileHeader) );
             If (OldH.PreviewMark <> 'PRV') Or (OldH.Version > 300)
               Then Begin
                      raise Exception.Create('Format error');
                    End;
             Header.PreviewMark := 'PRV';
             Header.Version := 300;
             If IsAdd
               Then Header.ItemsCount := OldH.ItemsCount + SizesList.Count + 1
               Else Header.ItemsCount := OldH.ItemsCount + SizesList.Count;
             Header.DataPos := SizeOf( TFileHeader );
             FF.Seek( 0, soFromBeginning );
             FF.Write( Header, SizeOf( TFileHeader ) );
             FF.Seek( 0, soFromEnd );
             ReCountDataPos;
             If IsAdd
               Then Begin
                     Empty.DataPos := Header.DataPos;
                     FF.Write( Empty, SizeOf(TStruct) );
                    End;
             For I := 0 To SizesList.Count-1 Do
               Begin
                 Item := SizesList.Items[I];
                 If (Item^.PageNumber = Page) Or (Item^.PageNumber = AllPages)
                   Then Begin
                          MemoryPaper.Seek( Item^.DataPos, soFromBeginning );
                          Item^.DataPos := FF.Position + SizeOf(TStruct);
                          FF.Write( Item^, SizeOf(TStruct) );
                          If Item^.HeaderSize + Item^.DataSize > 0
                            Then FF.CopyFrom( MemoryPaper, Item^.HeaderSize + Item^.DataSize );
                        End;  
               End;
           Finally
             FF.Free;
           End;
         End;
End;

Procedure TPaper.LoadPageFromFile( FileName : String; Page : SmallInt; ReadMethod : TRWMethod );
Var
  FF     : TFileStream;
  Header : TFileHeader;
  I      : Integer;
  Item   : ^TStruct;
Begin
  FF := TFileStream.Create( FileName, fmOpenRead );
  Try
    FF.Seek( 0, soFromBeginning );
    FF.Read( Header, SizeOf(TFileHeader) );
    If (Header.PreviewMark <> 'PRV') Or (Header.Version > 300)
      Then Begin
             raise Exception.Create('Format error');
           End;
    FF.Seek( Header.DataPos, soFromBeginning );
    If ReadMethod = PAGE_NEW
      Then Begin
             Clear;
             If Header.PaperFormat = pfCustom
               Then Begin
                      PrinterSetup.PaperWidth_MM := Header.PaperWidth;
                      PrinterSetup.PaperHeight_MM := Header.PaperHeight;
                    End
               Else Begin
                      PrinterSetup.PaperFormat := Header.PaperFormat;
                    End;
             Self.Units := Header.Units;
             PrinterSetup.Orientation := Header.Orientation;
           End;
    SizesList.Capacity := SizesList.Count + Header.ItemsCount;
    MemoryPaper.Position := MemoryPaper.Size;
    For I := 0 To Header.ItemsCount-1 Do
      Begin
        New( Item );
        FF.Read( Item^, SizeOf(TStruct) );
        If (Item^.PageNumber = Page) Or (Item^.PageNumber = AllPages)
          Then Begin
                 If Item^.HeaderSize + Item^.DataSize > 0
                   Then MemoryPaper.CopyFrom( FF, Item^.HeaderSize + Item^.DataSize );
                 SizesList.Add( Item );
               End
          Else Begin
                 Dispose( Item );
               End;
      End;
    ReCountDataPos;
  Finally
    FF.Free;
  End;
End;

Procedure TPaper.SetPage( Page : SmallInt );
Begin
  ActivePage := Page;
End;

Function TPaper.GetPage : SmallInt;
Begin
  Result := ActivePage;
End;

Procedure TPaper.ExportPageToFile( FileName : String; PageNumber : SmallInt; ImageFormat : TImageFormat );
Var
  Zoom1, Zoom2 , Zoom : Real;
  MetaFile            : TMetaFile;
  MetaFileCanvas      : TMetaFileCanvas;
  B : TBitmap;
  J : TJPEGImage;
Begin
  Zoom1 := Width / Printer.PageWidth;
  Zoom2 := Height / Printer.PageHeight;
  Zoom := (Zoom1 + Zoom2) / 2;
  Case ImageFormat Of
  ifBitmap : Begin
               B := TBitmap.Create;
               B.PixelFormat := pf24Bit;
               Try
                 B.Width := Width;
                 B.Height := Height;
                 DrawToCanvas( B.Canvas, Zoom, 100, PageNumber );
                 B.SaveToFile( FileName );
               Finally
                 B.Free;
               End;
             End;
  ifJPEG   : Begin
               B := TBitmap.Create;
               B.PixelFormat := pf24Bit;
               J := TJPEGImage.Create;
               J.PixelFormat := jf24Bit;
               Try
                 B.Width := Width;
                 B.Height := Height;
                 DrawToCanvas( B.Canvas, Zoom, 100, PageNumber );
                 J.Assign( B );
                 J.CompressionQuality := CompressionQuality;
                 J.SaveToFile( FileName );
               Finally
                 B.Free;
                 J.Free;
               End;
             End;
ifMetaFile : Begin
               MetaFile := TMetaFile.Create;
               MetaFile.Width := Width;
               MetaFile.Height := Height;
               Try
                 MetaFileCanvas := TMetaFileCanvas.Create( MetaFile, 0 );
                 Try
                   DrawToCanvas( MetaFileCanvas, Zoom, 100, PageNumber );
                 Finally
                   MetaFileCanvas.Free;
                 End;  
                 MetaFile.SaveToFile( FileName );
               Finally
                 MetaFile.Free;
               End;
             End;
  End;{ case }
End;


end.
