unit Preview;

interface

uses
  Windows, SysUtils, Messages, Graphics, ExtCtrls, Printers, Buttons, Classes,
  Controls, Forms, Dialogs, Paper, DIB_Image;

Const
  Max_Scale = 999; // 

Type
  TScale = 1..Max_Scale; //    1%  999%

  TFitScale = (fitPageWidth, fitPageHeight, fitWholePage );

  TLanguage = ( English, Russian );

  TButtons = ( btPrint, btPageSetup, btOpen, btSave, btHelp, btAbout, btLanguage );
  TButtonsSet = Set Of TButtons;

  TPrintNotifyEvent = procedure( Sender : TObject; PageNumber : SmallInt ) of Object;

{ TCustomPreviewForm }

Type
  TPreview = Class;
  
  TCustomPreviewForm = class(TForm)
  private
    { Private declarations }
  Protected
    { Protected declarations }
  public
    { Public declarations }
    _Preview   : TPreview;
    _Paper     : TPaper;
    WorkRect   : TRect;
  published
    { Published declarations }
  end;


{ TPreview }

  TPreview = class( TComponent )
  private
    { Private declarations }
    FPaper       : TPaper;
    FMaximumSize : Integer;//    
    FMax_Scale   : TScale;//    
    FMaxScale    : TScale; //     
    FMinScale    : TScale; //     
    FScale       : TScale;
    FStepScale   : TScale;
    FMinPage     : SmallInt;
    FMaxPage     : SmallInt;
    FCurrentPage : SmallInt;
    FButtons     : TButtonsSet;
    FPreviewVisible : Boolean;
    FLanguage    : TLanguage;
    {events}
    FOnPreview   : TNotifyEvent;
    FOnPrinting  : TPrintNotifyEvent;
    FOnClose     : TNotifyEvent;
    FChangeScale : TNotifyEvent;
    FChangePage  : TNotifyEvent;
    Procedure SetMaximumSize( Value : Integer );
    Procedure SetPaper( Value : TPaper );
    Procedure SetMaxScale( Value : TScale );
    Procedure SetMinScale( Value : TScale );
    Procedure SetFScale( Value : TScale );
    Procedure SetStepScale( Value : TScale );
    Procedure SetFMinPage( Value : SmallInt );
    Procedure SetFMaxPage( Value : SmallInt );
    Procedure SetFCurrentPage( Value : SmallInt );
    Procedure SetButtons( Value : TButtonsSet );
    Procedure SetLanguage( Value : TLanguage );
  protected
    { Protected declarations }
    Procedure PaperChangeSize( Sender : TObject );
  public
    { Public declarations }
    PreviewForm : TCustomPreviewForm;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    Procedure Preview;
    Procedure Print;
    Procedure PrintPage( PageNumber : SmallInt );
    Procedure PrintPages( FromPage, ToPage : SmallInt );
    Procedure PrintEx( PageNumber : SmallInt; Scale : TScale; Copies : Integer );
    Procedure FitPage( Fit : TFitScale );
    Procedure ZoomIn;
    Procedure ZoomOut;
  published
    { Published declarations }
    Property Paper       : TPaper Read FPaper Write SetPaper;
    Property MaximumSize : Integer Read FMaximumSize Write SetMaximumSize Default 6000;{ pixels }
    Property MaxScale    : TScale Read FMaxScale Write SetMaxScale Default 100;{ % }
    Property MinScale    : TScale Read FMinScale Write SetMinScale Default 25;{ % }
    Property Scale       : TScale Read FScale Write SetFScale;
    Property StepScale   : TScale Read FStepScale Write SetStepScale;
    Property MaxPage     : SmallInt Read FMaxPage Write SetFMaxPage Default 1;
    Property MinPage     : SmallInt Read FMinPage Write SetFMinPage Default 1;
    Property CurrentPage : SmallInt Read FCurrentPage Write SetFCurrentPage Default 1;
    Property Buttons     : TButtonsSet read FButtons write SetButtons;
    Property PreviewVisible : Boolean Read FPreviewVisible Default False;
    Property Language    : TLanguage Read FLanguage Write SetLanguage;
    { events }
    property OnPreview      : TNotifyEvent read FOnPreview write FOnPreview;
    property OnPrinting     : TPrintNotifyEvent read FOnPrinting write FOnPrinting;
    property OnChangePage   : TNotifyEvent read FChangePage write FChangePage;
    Property OnChangeScale  : TNotifyEvent Read FChangeScale Write FChangeScale;
    Property OnClosePreview : TNotifyEvent Read FOnClose Write FOnClose;
  end;

{********************************}

  Procedure WriteToLog( Msg : String );

{********************************}

implementation
Uses PreviewForm;

{****************************}
Procedure WriteToLog( Msg : String );
Var
  F : TEXT;
Begin
  Assign( F, 'ErrorLog.txt' );
  If FileExists('ErrorLog.txt')
    Then Begin
           Try
             Append( F );
           Except
             Exit;
           End
         End
    Else Begin
           Try
             ReWrite( F );
           Except

            Exit;
           End;
         End;
  WriteLn( F, DateTimeToStr( Now ) + ' : ' + Msg );
  CloseFile( F );
End;


{ =========== TPreview ============= }

{ Private }

Procedure TPreview.SetPaper( Value : TPaper );
Begin
  If FPaper <> Value
    Then Begin
           FPaper := Value;
         End;
End;

Procedure TPreview.SetMaximumSize( Value : Integer );
Var
  Z1, Z2 : Real;
Begin
  If FMaximumSize <> Value
    Then Begin
           FMaximumSize := Value;
           Z1 := FMaximumSize/Paper.Width;
           Z2 := FMaximumSize/Paper.Height;
           If Z2 > Z1
             Then FMax_Scale := Round( Z1 * 100 )
             Else FMax_Scale := Round( Z2 * 100 );
           If MaxScale > FMax_Scale
             Then FMaxScale := FMax_Scale;
         End;
End;

Procedure TPreview.SetMaxScale( Value : TScale );
Begin
  If (Value <> FMaxScale)
    Then Begin
           If Value <= FMax_Scale
             Then FMaxScale := Value
             Else FMaxScale := FMax_Scale;
         End;
End;

Procedure TPreview.SetMinScale( Value : TScale );
Begin
  If (FMinScale <> Value) And (Value <= MaxScale)
    Then Begin
           FMinScale := Value;
           If Assigned( FChangeScale )
             Then FChangeScale( Self );
         End;
End;

Procedure TPreview.SetFScale( Value : TScale );
Begin
  If (MinScale <= Value ) And (Value <= MaxScale) And (Value <> FScale)
    Then Begin
           FScale := Value;
           If Assigned( FChangeScale )
             Then FChangeScale( Self );
         End;
End;

Procedure TPreview.SetStepScale( Value : TScale );
Begin
  If (FStepScale <> Value) And (Value <= MaxScale) And (Value >= MinScale)
    Then Begin
           FStepScale := Value;
           If Assigned(FChangeScale)
             Then Self.FChangeScale( Self );
         End;
End;

Procedure TPreview.SetFCurrentPage( Value : SmallInt );
Begin
  If (MinPage <= Value) And (Value <= MaxPage) And (CurrentPage <> Value)
   Then Begin
          FCurrentPage := Value;
          If Assigned( Self.FChangePage )
            Then FChangePage( Self );
        End;
End;

Procedure TPreview.SetFMinPage( Value : SmallInt );
Begin
  If Value <= MaxPage
    Then If FMinPage <> Value
           Then Begin
                  FMinPage := Value;
                  If CurrentPage < FMinPage
                    Then FCurrentPage := FMinPage;
                End;
End;

Procedure TPreview.SetFMaxPage( Value : SmallInt );
Begin
  If Value >= MinPage
    Then If FMaxPage <> Value
           Then Begin
                  FMaxPage := Value;
                  If CurrentPage > MaxPage
                    Then FCurrentPage := MaxPage;
                End;
End;

Procedure TPreview.SetButtons( Value : TButtonsSet );
Begin
  If FButtons <> Value
    Then FButtons := Value;
End;

Procedure TPreview.SetLanguage( Value : TLanguage );
Begin
  If FLanguage <> Value
    Then FLanguage := Value;
End;

{ Protected }

Procedure TPreview.PaperChangeSize( Sender : TObject );
Var
  Z1, Z2 : Real;
Begin
  Z1 := FMaximumSize/Paper.Width;
  Z2 := FMaximumSize/Paper.Height;
  If Z2 > Z1
    Then FMax_Scale := Round( Z1 * 100 )
    Else FMax_Scale := Round( Z2 * 100 );
  If MaxScale > FMax_Scale
    Then FMaxScale := FMax_Scale;
  If Scale > MaxScale
    Then FScale := MaxScale;
End;

{ Public }

constructor TPreview.Create(AOwner: TComponent);
Begin
  inherited Create( AOwner );
  FMaximumSize := 6000;
  FMax_Scale := 750;
  FMaxScale := 750;
  FMinScale := 25;
  FScale := 100;
  FStepScale := 25;
  FCurrentPage := 1;
  FMinPage := 1;
  FMaxPage := 1;
  FButtons := [ btPrint, btPageSetup, btSave, btOpen, btAbout, btHelp, btLanguage ];
  Try
    Paper := TPaper.Create;
  Except
    WriteToLog('Unit <Preview.pas>: Error in creating object "Paper"');
    Exit;
  End;
  Try
    fmPreview := TfmPreview.Create( Self );
  Except
    WriteToLog('Unit <Preview.pas>: Error in creating object "fmPreview"');
    Exit;
  End;
  Paper.OnChangeSize := PaperChangeSize;
  PreviewForm := fmPreview;
  FLanguage := English;
End;

destructor TPreview.Destroy;
Begin
  fmPreview.Free;
  Paper.Free;
  inherited Destroy;
End;

Procedure TPreview.Print;
Var
  I : SmallInt;
Begin
  Printer.Title := 'Preview printing';
  For I := MinPage To MaxPage Do
    Begin
      Printer.BeginDoc;
      Paper.DrawToCanvas( Printer.Canvas, 1, 100, I );
      If Assigned(FOnPrinting )
        Then FOnPrinting( Self, I );
      Printer.EndDoc;
    End;
End;

Procedure TPreview.PrintPage( PageNumber : SmallInt );
Begin
  Printer.Title := 'Preview printing';
  Printer.BeginDoc;
  Paper.DrawToCanvas( Printer.Canvas, 1, 100, PageNumber );
  If Assigned(FOnPrinting )
    Then FOnPrinting( Self, PageNumber );
  Printer.EndDoc;
End;

Procedure TPreview.PrintPages( FromPage, ToPage : SmallInt );
Var
  I : SmallInt;
Begin
  Printer.Title := 'Preview printing';
  For I := FromPage To ToPage Do
    Begin
      Printer.BeginDoc;
      Paper.DrawToCanvas( Printer.Canvas, 1, 100, I );
      If Assigned(FOnPrinting )
        Then FOnPrinting( Self, I );
      Printer.EndDoc;
    End;
End;

Procedure TPreview.PrintEx( PageNumber : SmallInt; Scale : TScale; Copies : Integer );
Begin
  Printer.Title := 'Preview printing';
  Printer.Copies := Copies;
  Printer.BeginDoc;
  Paper.DrawToCanvas( Printer.Canvas, 1, Scale, PageNumber );
  If Assigned(FOnPrinting )
    Then FOnPrinting( Self, PageNumber );
  Printer.EndDoc;
End;

Procedure TPreview.FitPage( Fit : TFitScale );
Const
  FitOffSet = 2;
Var
  Scale1 : SmallInt;
  Scale2 : SmallInt;
Begin
  If PreviewVisible
    Then Begin
           Case Fit Of
           fitPageWidth  : Begin
                             Scale := Round( (PreviewForm.WorkRect.Right - PreviewForm.WorkRect.Left - FitOffSet ) / Paper.Width*100);
                           End;
           fitPageHeight : Begin
                             Scale := Round( (PreviewForm.WorkRect.Bottom - PreviewForm.WorkRect.Top - FitOffSet ) / Paper.Height*100);
                           End;
           fitWholePage  : Begin
                             Scale1 := Round( (PreviewForm.WorkRect.Right - PreviewForm.WorkRect.Left - FitOffSet ) / Paper.Width*100 );
                             Scale2 := Round( (PreviewForm.WorkRect.Bottom - PreviewForm.WorkRect.Top - FitOffSet ) / Paper.Height*100 );
                             If Scale1 < Scale2
                               Then Scale := Scale1
                               Else Scale := Scale2;
                           End;
           End;{ case }
         End;
End;

Procedure TPreview.Preview;
Begin
  If Assigned( PreviewForm )
    Then Begin
           Try
             PreviewForm._Preview := Self;
             PreviewForm._Paper := Paper;
             FPreviewVisible := True;
             If Assigned( FOnPreview )
               Then FOnPreview( Self );
             PreviewForm.ShowModal;
             If Assigned( FOnClose )
               Then FOnClose( Self );
           Finally
             FPreviewVisible := False;
           End;
         End;
End;

Procedure TPreview.ZoomIn;
Var
  Old_Scale : Integer;
  Loc_Scale : Integer;
  Loc_Step  : Integer;
Begin
  Old_Scale := Scale;
  Loc_Scale := Scale;
  Loc_Step := StepScale;
  Loc_Scale := Loc_Scale + Loc_Step;
  If Loc_Scale > MaxScale
    Then Loc_Scale := MaxScale
    Else Loc_Scale := ( Trunc( Loc_Scale/Loc_Step ) ) * Loc_Step;
  If Old_Scale <> Loc_Scale
    Then Begin
           Scale := Loc_Scale;
         End;
End;

Procedure TPreview.ZoomOut;
Var
  Old_Scale : Integer;
  Loc_Scale : Integer;
  Loc_Step  : Integer;
Begin
  Old_Scale := Scale;
  Loc_Scale := Scale;
  Loc_Step := StepScale;
  Loc_Scale := Loc_Scale - Loc_Step;
  If Loc_Scale < MinScale
    Then Loc_Scale := MinScale
    Else Loc_Scale := ( Round( Loc_Scale/Loc_Step + 0.49 ) ) * Loc_Step;
  If Old_Scale <> Loc_Scale
    Then Begin
           Scale := Loc_Scale;
         End;
End;

end.





