unit PrinterOptionsEd;

interface

uses
  Classes, Windows, Forms, DsgnIntf, StdCtrls, Graphics, Controls, ExtCtrls,
  Buttons, ComCtrls, Messages, Paper, Printers;

type
  TfmPrinterOptions = class(TForm)
    Image1: TImage;
    Label1: TLabel;
    Label2: TLabel;
    Bevel1: TBevel;
    Label3: TLabel;
    GroupBox1: TGroupBox;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    cbFormat: TComboBox;
    Label5: TLabel;
    edWidthMM: TEdit;
    edHeightMM: TEdit;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    edWidthInch: TEdit;
    Label11: TLabel;
    edHeightInch: TEdit;
    PixelsPerMillimeter: TLabel;
    stByX: TStaticText;
    stByY: TStaticText;
    Label12: TLabel;
    Label13: TLabel;
    Panel1: TPanel;
    LabelA: TLabel;
    ShapePaper: TShape;
    bbCancel: TButton;
    bbOK: TButton;
    procedure Label3MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure OnMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure Label3Click(Sender: TObject);
    procedure edWidthMMKeyPress(Sender: TObject; var Key: Char);
    procedure RadioButton1Click(Sender: TObject);
    procedure RadioButton2Click(Sender: TObject);
    procedure edWidthMMExit(Sender: TObject);
    procedure edHeightMMExit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbFormatClick(Sender: TObject);
  private
    { Private declarations }
    FPrinterSetup : TPrinterSetup;
    Procedure SetPrinterSetup( Value : TPrinterSetup );
  Protected
  public
    { Public declarations }
    Procedure SetPaperFormat( Value : TPaperFormat );
    Procedure SetOrientation( Value : TPrinterOrientation );
    Procedure SetWidth_MM( Value : Integer );
    Procedure SetHeight_MM( Value : Integer );
  published
    Property CurrPrinter : TPrinterSetup Read FPrinterSetup Write SetPrinterSetup;
  end;


Var
  fmPrinterOptions : TfmPrinterOptions;

implementation
Uses SysUtils, ShellAPI;
{$R *.DFM}

Procedure TfmPrinterOptions.SetPaperFormat( Value : TPaperFormat );
Begin
  cbFormat.ItemIndex := cbFormat.Items.IndexOf( CurrPrinter.GetFormatName(CurrPrinter.PaperFormat) );
End;

Procedure TfmPrinterOptions.SetOrientation( Value : TPrinterOrientation );
Begin
  Case Value Of
  poPortrait  : Begin
                  RadioButton1.Checked := True;
                  With ShapePaper Do
                    Begin
                      Left := 13;
                      Top := 7;
                      Width := 27;
                      Height := 35;
                      LabelA.Left := 15;
                      LabelA.Top := 7;
                    End;
                End;
  poLandScape : Begin
                  RadioButton2.Checked := True;
                  With ShapePaper Do
                    Begin
                      Left := 12;
                      Top := 16;
                      Width := 35;
                      Height := 27;
                      LabelA.Left := 15;
                      LabelA.Top := 18;
                    End;
                End;
  End;{Case}
End;

Procedure TfmPrinterOptions.SetWidth_MM( Value : Integer );
Begin
  edWidthMM.Text := IntToStr( CurrPrinter.PaperWidth_MM );
  edWidthInch.Text := FloatToStr( CurrPrinter.PaperWidth_Inch );
  If CurrPrinter.PaperWidth_MM <> 0
    Then stByX.Caption := FloatToStr(CurrPrinter.PixelsPerMM_byX);
End;

Procedure TfmPrinterOptions.SetHeight_MM( Value : Integer );
Begin
  edHeightMM.Text := IntToStr( CurrPrinter.PaperHeight_MM );
  edHeightInch.Text := FloatToStr( CurrPrinter.PaperHeight_Inch );
  If CurrPrinter.PaperHeight_MM <> 0
    Then stByY.Caption := FloatToStr(CurrPrinter.PixelsPerMM_byY);
End;

Procedure TfmPrinterOptions.SetPrinterSetup( Value : TPrinterSetup );
Begin
  If FPrinterSetup <> Value
    Then FPrinterSetup := Value;
End;

procedure TfmPrinterOptions.Label3MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  Label3.Font.Color := $00F57223;
  Label3.Font.Style := [fsUnderline];
end;

procedure TfmPrinterOptions.OnMouseLeave(var Message: TMessage);
begin
  Label3.Font.Color := clBlue;
  Label3.Font.Style := [];
end;

procedure TfmPrinterOptions.Label3Click(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  Try
    ShellExecute( GetDesktopWindow(), 'open',
                  PChar('mailto:rustamkafarov@hotmail.com'),
                  nil, nil, SW_ShowNormal );
  Except
  End;
  Screen.Cursor := crDefault;
end;

procedure TfmPrinterOptions.RadioButton1Click(Sender: TObject);
begin
  CurrPrinter.Orientation := poPortrait;
  SetOrientation( CurrPrinter.Orientation );
  SetWidth_MM( CurrPrinter.PaperWidth_MM );
  SetHeight_MM( CurrPrinter.PaperHeight_MM );
end;

procedure TfmPrinterOptions.RadioButton2Click(Sender: TObject);
begin
  CurrPrinter.Orientation := poLandscape;
  SetOrientation( CurrPrinter.Orientation );
  SetWidth_MM( CurrPrinter.PaperWidth_MM );
  SetHeight_MM( CurrPrinter.PaperHeight_MM );
end;

procedure TfmPrinterOptions.edWidthMMKeyPress(Sender: TObject;
  var Key: Char);
Var
  N : Integer;
  S : String;
  I : TPaperFormat;
  P : TPaperFormat;
begin
  If Not( Key In ['0'..'9', #8, #13, #27] )
    Then Key := #0;
  Case Key Of
  #13 : Begin
          S := (Sender As TEdit).Text;
          Try
            N := StrToInt( Trim(S) );
          Except
            Case (Sender As TEdit).Tag Of
            1 : SetWidth_MM( CurrPrinter.PaperWidth_MM );
            2 : SetHeight_MM( CurrPrinter.PaperHeight_MM );
            End;{ case }
            Exit;
          End;
          Case (Sender As TEdit).Tag Of
          1 : CurrPrinter.PaperWidth_MM := N;
          2 : CurrPrinter.PaperHeight_MM := N;
          End;{ case }
          P := CurrPrinter.PaperFormat;
          For I := Low(TPaperFormat) To High(TPaperFormat) Do
            Begin
              If CurrPrinter.Orientation = poPortrait
                Then Begin
                       If (PaperFormats[I].Width_MM = CurrPrinter.PaperWidth_MM) And
                          (PaperFormats[I].Height_MM = CurrPrinter.PaperHeight_MM)
                        Then Begin
                               P := I;
                               Break;
                             End;
                     End
                Else Begin
                       If (PaperFormats[I].Width_MM = CurrPrinter.PaperHeight_MM) And
                          (PaperFormats[I].Height_MM = CurrPrinter.PaperWidth_MM)
                        Then Begin
                               P := I;
                               Break;
                             End;
                     End;
            End;
          CurrPrinter.PaperFormat := P;
          SetPaperFormat( P );
          Case (Sender As TEdit).Tag Of
          1 : SetWidth_MM( CurrPrinter.PaperWidth_MM );
          2 : SetHeight_MM( CurrPrinter.PaperHeight_MM );
          End;{ case }
        End;
  #27 : Begin
          Case (Sender As TEdit).Tag Of
          1 : SetWidth_MM( CurrPrinter.PaperWidth_MM );
          2 : SetHeight_MM( CurrPrinter.PaperHeight_MM );
          End;{ case }
        End;
  End; { Case }
end;


procedure TfmPrinterOptions.edWidthMMExit(Sender: TObject);
begin
  SetWidth_MM( CurrPrinter.PaperWidth_MM );
end;

procedure TfmPrinterOptions.edHeightMMExit(Sender: TObject);
begin
  SetHeight_MM( CurrPrinter.PaperHeight_MM );
end;

procedure TfmPrinterOptions.FormCreate(Sender: TObject);
begin
  CurrPrinter := TPrinterSetup.Create;
end;

procedure TfmPrinterOptions.FormDestroy(Sender: TObject);
begin
  CurrPrinter.Free;
end;

procedure TfmPrinterOptions.cbFormatClick(Sender: TObject);
begin
  CurrPrinter.SetFormatByName( cbFormat.Text );
  SetPaperFormat( CurrPrinter.PaperFormat );
  SetWidth_MM( CurrPrinter.PaperWidth_MM );
  SetHeight_MM( CurrPrinter.PaperHeight_MM );
end;

end.
